# -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is mozilla.org code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Alec Flett         <alecf@netscape.com>
#   Ben Goodger        <ben@netscape.com>
#   Mike Pinkerton     <pinkerton@netscape.com>
#   Blake Ross         <blakeross@telocity.com>
#   Christopher Thomas <cst@yecc.com>
#   Postbox, Inc. <contact@postbox-inc.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

/*
 * - [ Dependencies ] ---------------------------------------------------------
 *  utilityOverlay.js:
 *    - gatherTextUnder
 */

  var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  
  /** 
   * extract the href from the link click event. 
   * We look for HTMLAnchorElement, HTMLAreaElement, HTMLLinkElement,
   * HTMLInputElement.form.action, and nested anchor tags.
   * 
   * @return href for the url being clicked
   */
  function hRefForClickEvent(event)
  {
    var target = event.target;
    var href;
    var isKeyPress = (event.type == "keypress");

    if (target instanceof HTMLAnchorElement ||
        target instanceof HTMLAreaElement   ||
        target instanceof HTMLLinkElement)
    {
      if (target.hasAttribute("href"))
      {
	if (target.href)
	  href = target.href;
        else {
          // hack, pre-pend the scheme
          // see bug #8348 for details
          href = "http://" + target.getAttribute("href");
        }
      }
    }
    else if (target instanceof HTMLInputElement)
    {
      if (target.form && target.form.action)
        href = target.form.action;      
    }
    else 
    {
      // we may be nested inside of a link node
      var linkNode = event.originalTarget;
      while (linkNode && !(linkNode instanceof HTMLAnchorElement))
        linkNode = linkNode.parentNode;
      
      if (linkNode)
        href = linkNode.href;
    }

    return href;
  }

  function adjustIframeHeightsOnPaint()
  {
    adjustIframeHeights(true /* aOnPaint */);
  }

  function adjustIframeHeights(aOnPaint)
  {
    var messagepane = document.getElementById("messagepane");
    var doc = messagepane.contentDocument;
    var dynamicStatusBar = document.getElementById("statusbar-display");

    var iframes = doc.getElementsByTagName('iframe');
    for (var i = 0; i < iframes.length; i++)
    { 
      if (iframes[i].className == "postbox-conversation-iframe")
      {
        AdjustConversationHeight(messagepane, iframes[i], iframes[i].contentDocument.body, aOnPaint, false /* aForceAdjustmentNow */);
      }
    }  
  }

  var gLastMessagePaneWidth = -1;
  var gLastMessagePaneHeight = -1;

  function messagePaneOnResize(event)
  {
    // scale any overflowing images
    var messagepane = document.getElementById("messagepane");
    
    // only resize images and adjust the iframe height when the messagepane <browser> resizes
    // we can also get called when and iframe resizes, and we don't want to resize on that.
    // see bug #7285 for details
    if (gLastMessagePaneWidth == messagepane.clientWidth && gLastMessagePaneHeight == messagepane.clientHeight)
      return;
    
    gLastMessagePaneWidth = messagepane.clientWidth;
    gLastMessagePaneHeight = messagepane.clientHeight;
    
    // XXX TODO
    // is this still necessary?  isn't everything in an iframe now?
    var doc = messagepane.contentDocument;
    ScaleOverflowingImagesOnResize(doc);
    
    var iframes = doc.getElementsByTagName('iframe');
    for (var i = 0; i < iframes.length; i++)
    { 
      if (iframes[i].className == "postbox-conversation-iframe")
      {
        ScaleOverflowingImagesOnResize(iframes[i].contentDocument);
        
        // we need force the document to resize based on the new height of this iframe.
        // see bug #7212 for details
        AdjustConversationHeight(messagepane, iframes[i], iframes[i].contentDocument.body, false /* aOnPaint */, false /* aForceAdjustmentNow */);
      }
    }
  }

  function ScaleOverflowingImagesOnResize(doc)
  {
    var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                       getService(Components.interfaces.nsIPrefService).getBranch(null);
    var shrinkToFit = prefBranch.getBoolPref("mail.enable_automatic_image_resizing");

    var imgs = doc.images;
    for (var i=0; i < imgs.length; i++)
    {
      var img = imgs[i];
      if (img.naturalWidth <= doc.body.clientWidth)
      {
        img.removeAttribute("isshrunk");
        img.removeAttribute("overflowing");
      }
      else if (shrinkToFit)
      {
        // the user might have shrink to fit enabled, but disabled it for this image (by clicking on it)
        // in that case, don't set isshrunk on resizing the window
        var shrinkToFitEnabledForImage = img.getAttribute("shrinktofit") != "false";
        if (shrinkToFitEnabledForImage) {
          img.setAttribute("isshrunk", "true");
          img.removeAttribute("overflowing");
        } else {
          img.setAttribute("overflowing", "true");
          img.removeAttribute("isshrunk");
        }
      }
      else
      {
        img.setAttribute("overflowing", "true");
        img.removeAttribute("isshrunk");
        img.setAttribute("shrinktofit", "false");
        img.setAttribute("resizecursor", "default");
      }
    }
  }

  // Called whenever the user clicks in the content area,
  // except when left-clicking on links (special case)
  // should always return true for click to go through
  function contentAreaClick(event) 
  {
    var href = hRefForClickEvent(event);
    if (href) 
    {
      handleLinkClick(event, href, null);
      if (!event.button)  // left click only
        return gPhishingDetector.warnOnSuspiciousLinkClick(href); // let the phishing detector check the link
    }
    else if (!event.button)
    {
      var target = event.target;
      // is this an image that we might want to scale?
      const Ci = Components.interfaces;
      if (target instanceof Ci.nsIImageLoadingContent) 
      {
        // make sure it loaded successfully
        var req = target.getRequest(Ci.nsIImageLoadingContent.CURRENT_REQUEST);
        if (!req || req.imageStatus & Ci.imgIRequest.STATUS_ERROR)
          return true;

        var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                           getService(Components.interfaces.nsIPrefService).getBranch(null);
        var shrinkToFit = prefBranch.getBoolPref("mail.enable_automatic_image_resizing");

        if (shrinkToFit)
        {
          if (target.hasAttribute("isshrunk"))
          {
            // currently shrunk to fit, so unshrink it
            target.removeAttribute("isshrunk");
            target.setAttribute("overflowing", "true");
            // since the user unshrunk it, set this so we don't shrink on resize
            target.setAttribute("shrinktofit", "false");
          }
          else if (target.hasAttribute("overflowing"))
          {
            // user wants to shrink now
            target.setAttribute("isshrunk", "true");
            target.removeAttribute("overflowing");
            // since the user shrunk it, remove this so we shrink on resize
            target.removeAttribute("shrinktofit");
          }
          
          // on image resize, adjust the iframe heights
          // see bug #6629 for details
          adjustIframeHeights(false /* aOnPaint */);
        }
      }
    }
    
    return true;
  }

  function openNewTabOrWindow(event, href, sendReferrer)
  {
    // always return false for stand alone mail (MOZ_THUNDERBIRD)
    // let someone else deal with it
    return false;
  }

  function getContentFrameURI(aFocusedWindow)
  {
    var contentFrame = isContentFrame(aFocusedWindow) ? aFocusedWindow : window.content;
    return contentFrame.location.href;
  }

  function handleLinkClick(event, href, linkNode)
  {
    // Make sure we are allowed to open this URL
    var focusedWindow = document.commandDispatcher.focusedWindow;
    var sourceURL = getContentFrameURI(focusedWindow);
    urlSecurityCheck(href, sourceURL);
    return false;
  }

  function middleMousePaste( event )
  {
    return false;
  }

  function makeURLAbsolute( base, url ) 
  {
    // Construct nsIURL.
    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
    var baseURI  = ioService.newURI(base, null, null);

    return ioService.newURI(baseURI.resolve(url), null, null).spec;
  }

/**
 * Forces a url to open in an external application according to the protocol
 * service settings.
 *
 * @param url  A url string or an nsIURI containing the url to open.
 */
function openLinkExternally(url)
{
  let uri = url;
  if (!(uri instanceof Components.interfaces.nsIURI))
    uri = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService)
                    .newURI(url, null, null);

  Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
            .getService(Components.interfaces.nsIExternalProtocolService)
            .loadUrl(uri);
}

#include pbContentAreaClick.js
