# -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Communicator client code, released
# March 31, 1998.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998-1999
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   timeless
#   slucy@objectivesw.co.uk
#   Håkan Waara <hwaara@chello.se>
#   Jan Varga <varga@nixcorp.com>
#   Seth Spitzer <sspitzer@netscape.com>
#   David Bienvenu <bienvenu@nventure.com>
#   Karsten Düsterloh <mnyromyr@tprac.de>
#   Christopher Thomas <cst@yecc.com>
#   Jeremy Morton <bugzilla@game-point.net>
#   Postbox, Inc. <contact@postbox-inc.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

const Cc = Components.classes;
const Ci = Components.interfaces;

const MSG_FLAG_READ              = 0x000001;
const MSG_FLAG_MARKED            = 0x000004;
const MSG_FLAG_HAS_RE            = 0x000010;
const MSG_FLAG_IMAP_DELETED      = 0x200000;
const MSG_FLAG_MDN_REPORT_NEEDED = 0x400000;
const MSG_FLAG_MDN_REPORT_SENT   = 0x800000;
const MSG_FLAG_DRAFT             = 0x20000000;
const MSG_FLAG_NEW               = 0x10000;
const MSG_FLAG_PENDING           = 0x20000;
const MDN_DISPOSE_TYPE_DISPLAYED = 0;
const ADDR_DB_LARGE_COMMIT       = 1;
const MSG_DB_LARGE_COMMIT        = 1;

const kClassicMailLayout = 0;
const kWideMailLayout = 1;
const kVerticalMailLayout = 2;

// Per message header flags to keep track of whether the user is allowing remote
// content for a particular message. 
// if you change or add more values to these constants, be sure to modify
// the corresponding definitions in nsMsgContentPolicy.cpp
const kNoRemoteContentPolicy = 0;
const kBlockRemoteContent = 1;
const kAllowRemoteContent = 2;

var gMessengerBundle;
var gWindowManagerInterface;
var gPrefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService).getBranch(null);
var gMarkViewedMessageAsReadTimer = null; // if the user has configured the app to mark a message as read if it is viewed for more than n seconds
var gTimelineService = null;
var gTimelineEnabled = ("@mozilla.org;timeline-service;1" in Components.classes);
if (gTimelineEnabled) {
  try {
    gTimelineEnabled = gPrefBranch.getBoolPref("mailnews.timeline_is_enabled");
    if (gTimelineEnabled) {
      gTimelineService = 
        Components.classes["@mozilla.org;timeline-service;1"].getService(Components.interfaces.nsITimelineService);
    }
  }
  catch (ex)
  {
    gTimelineEnabled = false;
  }
}

var gShrinkToFit = true;
try {
  gShrinkToFit = gPrefBranch.getBoolPref("mail.enable_automatic_image_resizing");
} catch (ex) {
  // unexpected
}

// the user preference,
// if HTML is not allowed. I assume, that the user could have set this to a
// value > 1 in his prefs.js or user.js, but that the value will not
// change during runtime other than through the MsgBody*() functions below.
var gDisallow_classes_no_html = 1; 

// Disable the new account menu item if the account preference is locked.
// Two other affected areas are the account central and the account manager
// dialog.
function menu_new_init()
{
  if (!gMessengerBundle)
    gMessengerBundle = document.getElementById("bundle_messenger");

  var newAccountItem = document.getElementById('newAccountMenuItem');
  if (gPrefBranch.prefIsLocked("mail.disable_new_account_addition"))
    newAccountItem.setAttribute("disabled","true");

  // Change "New Folder..." menu according to the context
  var folderArray = GetSelectedMsgFolders();
  if (folderArray.length == 0)
    return;
  var msgFolder = folderArray[0];
  var isServer = msgFolder.isServer;
  var serverType = msgFolder.server.type;
  var canCreateNew = msgFolder.canCreateSubfolders;
  var isInbox = IsSpecialFolder(msgFolder, MSG_FOLDER_FLAG_INBOX, false);
  var isIMAPFolder = serverType == "imap";
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                         .getService(Components.interfaces.nsIIOService);
  var showNew = (serverType != 'nntp');
  ShowMenuItem("menu_newFolder", showNew);
  ShowMenuItem("menu_newVirtualFolder", showNew);

  EnableMenuItem("menu_newFolder", !isIMAPFolder || MailOfflineMgr.isOnline());
  EnableMenuItem("menu_newVirtualFolder", true);
  if (showNew)
    SetMenuItemLabel("menu_newFolder", gMessengerBundle.getString((isServer || isInbox || !canCreateNew) ? "newFolderMenuItem" : "newSubfolderMenuItem"));

  ShowMenuItem("menu_newAccountGroup", true);
  EnableMenuItem("menu_newAccountGroup", true);
}

function goUpdateMailMenuItems(commandset)
{
//  dump("Updating commands for " + commandset.id + "\n");
  for (var i = 0; i < commandset.childNodes.length; i++)
  {
    var commandID = commandset.childNodes[i].getAttribute("id");
    if (commandID)
      goUpdateCommand(commandID);
  }
}

function file_init()
{
  document.commandDispatcher.updateCommands('create-menu-file');
}

function InitEditMessagesMenu()
{
  goSetMenuValue('cmd_delete', 'valueDefault');
  goSetAccessKey('cmd_delete', 'valueDefaultAccessKey');
  document.commandDispatcher.updateCommands('create-menu-edit');

  // initialize the favorite Folder checkbox in the edit menu
  var favoriteFolderMenu = document.getElementById('menu_favoriteFolder');
  if (favoriteFolderMenu && !favoriteFolderMenu.disabled && FolderPaneSelectionIsMailFolder())
  {
    var folderTree = GetFolderTree();
    var startIndex = {};
    var endIndex = {};
    folderTree.view.selection.getRangeAt(0, startIndex, endIndex);
    if (startIndex.value >= 0)
    {
      var numSelected = endIndex.value - startIndex.value + 1;
      var folder = GetFolderResource(folderTree, startIndex.value);
      SetupFavoritesMenuItem(folder, numSelected, folder.isServer, 'menu_favoriteFolder');
    }
  }
}

function InitGoMessagesMenu()
{
  document.commandDispatcher.updateCommands('create-menu-go');
}

function view_init()
{
  if (!gMessengerBundle)
      gMessengerBundle = document.getElementById("bundle_messenger");
  var message_menuitem=document.getElementById('menu_showMessage');
  var threadPaneHidden = !IsThreadPaneVisible();

  if (message_menuitem)
  {
    var message_menuitem_hidden = message_menuitem.getAttribute("hidden");
    if(message_menuitem_hidden != "true")
    {
      message_menuitem.setAttribute('checked', !IsMessagePaneCollapsed());
      message_menuitem.setAttribute('disabled', threadPaneHidden);
    }
  }

  var focuspane_menuitem=document.getElementById("pbViewMenuShowFocusPane");
  if (focuspane_menuitem && !threadPaneHidden)
  {
    var focuspane_menuitem_hidden = focuspane_menuitem.getAttribute("hidden");
    if(focuspane_menuitem_hidden != "true")
    {
      focuspane_menuitem.setAttribute('checked', !IsFocusPaneCollapsed());
      focuspane_menuitem.setAttribute('disabled', threadPaneHidden);
    }
  }

  var folderpane_menuitem=document.getElementById("pbViewMenuShowFolderPane");
  if (folderpane_menuitem && !threadPaneHidden)
  {
    var folderpane_menuitem_hidden = folderpane_menuitem.getAttribute("hidden");
    if(folderpane_menuitem_hidden != "true")
    {
      folderpane_menuitem.setAttribute('checked', !IsFolderPaneCollapsed());
      folderpane_menuitem.setAttribute('disabled', threadPaneHidden);
    }
  }

  var pbViewMenuShowFavoritesBar=document.getElementById("pbViewMenuShowFavoritesBar");
  if (pbViewMenuShowFavoritesBar && !threadPaneHidden)
  {
    var pbViewMenuShowFavoritesBar_hidden = pbViewMenuShowFavoritesBar.getAttribute("hidden");
    if(pbViewMenuShowFavoritesBar_hidden != "true")
    {
      pbViewMenuShowFavoritesBar.setAttribute('checked', !document.getElementById('favoritesBarContainer').collapsed);
    }
  }


  var inspectorpane_menuitem=document.getElementById("pbViewMenuShowInspectorPane");
  if (inspectorpane_menuitem)
  {
    var inspectorpane_menuitem_hidden = inspectorpane_menuitem.getAttribute("hidden");
    if(inspectorpane_menuitem_hidden != "true")
    {
      inspectorpane_menuitem.setAttribute('checked', !IsInspectorPaneCollapsed());
    }
  }

  // Disable some menus if the thread pane isn't loaded
  document.getElementById('viewSortMenu').setAttribute("disabled", threadPaneHidden);
  var pinStarredMessagesMenu = document.getElementById('viewMenuPinStarredMessages');
  var showConversationsMenu = document.getElementById('viewMenuShowConversations');
  showConversationsMenu.setAttribute("disabled", threadPaneHidden);
  pinStarredMessagesMenu.setAttribute("disabled", threadPaneHidden);
  if (!threadPaneHidden)
  {
    showConversationsMenu.setAttribute('checked', (gDBView.viewFlags & nsMsgViewFlagsType.kThreadedDisplay) != 0);
    pinStarredMessagesMenu.setAttribute('checked', pref.getBoolPref('mail.pinstarred'));
  }
  var threads_menuitem = document.getElementById('viewMessagesMenu');
  if (threads_menuitem)
    threads_menuitem.setAttribute("disabled", threadPaneHidden);

  // Initialize the View Attachment Inline menu
  var viewAttachmentInline = pref.getBoolPref("mail.inline_attachments");
  document.getElementById("viewAttachmentsInlineMenuitem").setAttribute("checked", 
                          viewAttachmentInline ? "true" : "false");
                          
#ifdef XP_MACOSX
  document.getElementById('fullScreenMode').setAttribute('label', 
    gMessengerBundle.getString(window.infullscreenmode ? 'exitFullScreenMode' 
      : 'enterFullScreenMode'));  
#endif

  document.commandDispatcher.updateCommands('create-menu-view');
}

function InitViewLayoutStyleMenu(event)
{
  var paneConfig = pref.getIntPref("mail.pane_config.dynamic");
  if (paneConfig == kWideMailLayout)
    paneConfig = kClassicMailLayout;
  if (paneConfig == kClassicMailLayout)
    event.target.childNodes[0].setAttribute("checked", "true");
  else
    event.target.childNodes[1].setAttribute("checked", "true");
}

function setSortByMenuItemCheckState(id, value)
{
  var menuitem = document.getElementById(id);
  if (menuitem) 
    menuitem.setAttribute("checked", value);
}

function TogglePinStarredMessages(target)
{
  pref.setBoolPref("mail.pinstarred", !pref.getBoolPref("mail.pinstarred"));
}

function ToggleShowConversations(target)
{
  MsgToggleThreaded();
}

function InitViewSortByMenu()
{
  var sortType = gDBView.sortType;

  setSortByMenuItemCheckState("sortByDateMenuitem", (sortType == nsMsgViewSortType.byDate));
  setSortByMenuItemCheckState("sortByReceivedMenuitem", (sortType == nsMsgViewSortType.byReceived));
  setSortByMenuItemCheckState("sortByFlagMenuitem", (sortType == nsMsgViewSortType.byFlagged));
  setSortByMenuItemCheckState("sortByOrderReceivedMenuitem", (sortType == nsMsgViewSortType.byId));
  setSortByMenuItemCheckState("sortByPriorityMenuitem", (sortType == nsMsgViewSortType.byPriority));
  setSortByMenuItemCheckState("sortBySizeMenuitem", (sortType == nsMsgViewSortType.bySize));
  setSortByMenuItemCheckState("sortByStatusMenuitem", (sortType == nsMsgViewSortType.byStatus));
  setSortByMenuItemCheckState("sortBySubjectMenuitem", (sortType == nsMsgViewSortType.bySubject));
  setSortByMenuItemCheckState("sortByUnreadMenuitem", (sortType == nsMsgViewSortType.byUnread));
  setSortByMenuItemCheckState("sortByTagsMenuitem", (sortType == nsMsgViewSortType.byTags));
  setSortByMenuItemCheckState("sortByJunkStatusMenuitem", (sortType == nsMsgViewSortType.byJunkStatus));
  setSortByMenuItemCheckState("sortByFromMenuitem", (sortType == nsMsgViewSortType.byAuthor));
  setSortByMenuItemCheckState("sortByRecipientMenuitem", (sortType == nsMsgViewSortType.byRecipient)); 
  setSortByMenuItemCheckState("sortByAttachmentsMenuitem", (sortType == nsMsgViewSortType.byAttachments));

  var sortOrder = gDBView.sortOrder;
  var sortTypeSupportsGrouping = (sortType == nsMsgViewSortType.byAuthor 
      || sortType == nsMsgViewSortType.byDate || sortType == nsMsgViewSortType.byReceived || sortType == nsMsgViewSortType.byPriority
      || sortType == nsMsgViewSortType.bySubject || sortType == nsMsgViewSortType.byTags
      || sortType == nsMsgViewSortType.byRecipient || sortType == nsMsgViewSortType.byAccount
      || sortType == nsMsgViewSortType.byStatus || sortType == nsMsgViewSortType.byFlagged
      || sortType == nsMsgViewSortType.byAttachments);

  setSortByMenuItemCheckState("sortAscending", (sortOrder == nsMsgViewSortOrder.ascending));
  setSortByMenuItemCheckState("sortDescending", (sortOrder == nsMsgViewSortOrder.descending));
}

function InitViewMessagesMenu()
{
  var viewFlags = (gDBView) ? gDBView.viewFlags : 0;
  var viewType = (gDBView) ? gDBView.viewType : 0;

  var allMenuItem = document.getElementById("viewAllMessagesMenuItem");
  if(allMenuItem)
      allMenuItem.setAttribute("checked",  (viewFlags & nsMsgViewFlagsType.kUnreadOnly) == 0 && (viewType == nsMsgViewType.eShowAllThreads));

  var unreadMenuItem = document.getElementById("viewUnreadMessagesMenuItem");
  if(unreadMenuItem)
      unreadMenuItem.setAttribute("checked", (viewFlags & nsMsgViewFlagsType.kUnreadOnly) != 0);

  var theadsWithUnreadMenuItem = document.getElementById("viewThreadsWithUnreadMenuItem");
  if(theadsWithUnreadMenuItem)
      theadsWithUnreadMenuItem.setAttribute("checked", viewType == nsMsgViewType.eShowThreadsWithUnread);

  var watchedTheadsWithUnreadMenuItem = document.getElementById("viewWatchedThreadsWithUnreadMenuItem");
  if(watchedTheadsWithUnreadMenuItem)
      watchedTheadsWithUnreadMenuItem.setAttribute("checked", viewType == nsMsgViewType.eShowWatchedThreadsWithUnread);
  
  var ignoredTheadsMenuItem = document.getElementById("viewIgnoredThreadsMenuItem");
  if(ignoredTheadsMenuItem)
      ignoredTheadsMenuItem.setAttribute("checked", (viewFlags & nsMsgViewFlagsType.kShowIgnored) != 0);

  var showConversationsMenu = document.getElementById('viewMenuShowConversations');
  showConversationsMenu.setAttribute('checked', (viewFlags & nsMsgViewFlagsType.kThreadedDisplay) != 0);
}

function InitMessageMenu()
{
  var aMessage = GetFirstSelectedMessage();
  var isNews = false;
  if(aMessage)
    isNews = IsNewsMessage(aMessage);

  //We show reply to Newsgroups only for news messages.
  var replyNewsgroupMenuItem = document.getElementById("replyNewsgroupMainMenu");
  if(replyNewsgroupMenuItem)
    replyNewsgroupMenuItem.setAttribute("hidden", isNews ? "" : "true");

  //For mail messages we say reply. For news we say ReplyToSender.
  var replyMenuItem = document.getElementById("replyMainMenu");
  if(replyMenuItem)
    replyMenuItem.setAttribute("hidden", !isNews ? "" : "true");

  var replySenderMenuItem = document.getElementById("replySenderMainMenu");
  if(replySenderMenuItem)
    replySenderMenuItem.setAttribute("hidden", isNews ? "" : "true");

  // we only kill and watch threads for news
  var threadMenuSeparator = document.getElementById("threadItemsSeparator");
  if (threadMenuSeparator)
    threadMenuSeparator.setAttribute("hidden", isNews ? "" : "true");
  var killThreadMenuItem = document.getElementById("killThread");
  if (killThreadMenuItem)
    killThreadMenuItem.setAttribute("hidden", isNews ? "" : "true");
  var watchThreadMenuItem = document.getElementById("watchThread");
  if (watchThreadMenuItem)
    watchThreadMenuItem.setAttribute("hidden", isNews ? "" : "true");

  // disable the move and copy menus if there are no messages selected.
  // disable the move menu if we can't delete msgs from the folder
  var moveMenu = document.getElementById("moveMenu");
  var msgFolder = GetLoadedMsgFolder();
  if(moveMenu)
  {
    var enableMenuItem = aMessage && msgFolder && msgFolder.canDeleteMessages;
    moveMenu.setAttribute("disabled", !enableMenuItem);
  }

  var copyMenu = document.getElementById("copyMenu");
  if(copyMenu)
    copyMenu.setAttribute("disabled", !aMessage);
  
  initMoveToFolderAgainMenu(document.getElementById("moveToFolderAgain"));

  // Disable Forward as/Label menu items if no message is selected
  var forwardAsMenu = document.getElementById("forwardAsMenu");
  if(forwardAsMenu)
    forwardAsMenu.setAttribute("disabled", !aMessage);

  var labelMenu = document.getElementById("labelMenu");
  if(labelMenu)
    labelMenu.setAttribute("disabled", !aMessage);

  // Disable mark menu when we're not in a folder
  var markMenu = document.getElementById("markMenu");
  if(markMenu)
    markMenu.setAttribute("disabled", !msgFolder);

  document.commandDispatcher.updateCommands('create-menu-message');
}

// initMoveToFolderAgainMenu
// enables / disables aMenuItem based on the value of mail.last_msg_movecopy_target_uri
// Adjusts the label and accesskey for aMenuItem to include the folder name
function initMoveToFolderAgainMenu(aMenuItem)
{
  var lastFolderURI = pref.getCharPref("mail.last_msg_movecopy_target_uri");
  var isMove = pref.getBoolPref("mail.last_msg_movecopy_was_move");  
  if (lastFolderURI)
  {
    var destResource = RDF.GetResource(lastFolderURI);
    var destMsgFolder = destResource.QueryInterface(Components.interfaces.nsIMsgFolder);
    aMenuItem.label = gMessengerBundle.getFormattedString(isMove ? 
                                                          "moveToFolderAgain" : "copyToFolderAgain",
                                                          [destMsgFolder.prettyName], 1);
    aMenuItem.setAttribute('accesskey', 
                           gMessengerBundle.getString(isMove ? 
                                                      "moveToFolderAgainAccessKey" : "copyToFolderAgainAccessKey"));
  }
}

function InitViewBodyMenu()
{
  var html_as = 0;
  var prefer_plaintext = false;
  var disallow_classes = 0;
  try
  {
    prefer_plaintext = pref.getBoolPref("mailnews.display.prefer_plaintext");
    html_as = pref.getIntPref("mailnews.display.html_as");
    disallow_classes = pref.getIntPref("mailnews.display.disallow_mime_handlers");
    if (disallow_classes > 0)
      gDisallow_classes_no_html = disallow_classes;
    // else gDisallow_classes_no_html keeps its inital value (see top)
  }
  catch (ex)
  {
    dump("failed to get the body plaintext vs. HTML prefs\n");
  }

  var AllowHTML_checked = false;
  var Sanitized_checked = false;
  var AsPlaintext_checked = false;
  if (!prefer_plaintext && !html_as && !disallow_classes)
    AllowHTML_checked = true;
  else if (!prefer_plaintext && html_as == 3 && disallow_classes > 0)
    Sanitized_checked = true;
  else if (prefer_plaintext && html_as == 1 && disallow_classes > 0)
    AsPlaintext_checked = true;
  // else (the user edited prefs/user.js) check none of the radio menu items

  var AllowHTML_menuitem = document.getElementById("bodyAllowHTML");
  var Sanitized_menuitem = document.getElementById("bodySanitized");
  var AsPlaintext_menuitem = document.getElementById("bodyAsPlaintext");
  if (AllowHTML_menuitem && Sanitized_menuitem && AsPlaintext_menuitem)
  {
    AllowHTML_menuitem.setAttribute("checked", AllowHTML_checked ? "true" : "false");
    Sanitized_menuitem.setAttribute("checked", Sanitized_checked ? "true" : "false");
    AsPlaintext_menuitem.setAttribute("checked", AsPlaintext_checked ? "true" : "false");
  }
  else
    dump("Where is my View|Body menu?\n");
}

function IsNewsMessage(messageUri)
{
  return (/^news-message:/.test(messageUri));
}

function IsImapMessage(messageUri)
{
  return (/^imap-message:/.test(messageUri));
}

function SetMenuItemLabel(menuItemId, customLabel)
{
  var menuItem = document.getElementById(menuItemId);
  if (menuItem)
    menuItem.setAttribute('label', customLabel);
}

function RemoveAllMessageTags()
{
  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                             .getService(Components.interfaces.nsIMsgTagService);
  var tagArray = tagService.getAllTags({});

  var allKeys = "";
  for (var j = 0; j < tagArray.length; ++j)
  {
    if (j)
      allKeys += " ";
    allKeys += tagArray[j].key;
  }

  ToggleMessageTag(allKeys, false /* addkey */);
}

function ToggleMessageTagKey(index)
{
  if (GetNumSelectedMessages() < 1)
    return;
  // set the tag state based upon that of the first selected message,
  // just like we do for markAsRead etc.
  var msgHdr = gDBView.hdrForFirstSelectedMessage;
  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                             .getService(Components.interfaces.nsIMsgTagService);
  var tagArray = tagService.getAllTags({});
  for (var i = 0; i < tagArray.length; ++i)
  {
    var key = tagArray[i].key;
    if (tagArray[i].active && !--index) 
    {
      // found the key, now toggle its state
      var curKeys = msgHdr.getStringProperty("keywords");
      if (msgHdr.label)
        curKeys += " $label" + msgHdr.label;
      var addKey  = (" " + curKeys + " ").indexOf(" " + key + " ") < 0;
      ToggleMessageTag(key, addKey);
      return;
    }
  }
}

function ToggleMessageTagMenu(target)
{
  var key    = target.getAttribute("value");
  var addKey = target.hasAttribute("addkey");
  ToggleMessageTag(key, addKey);
}

function MessageNotInArray(aMsgHdr, aMsgHdrArray)
{
  var messageId = aMsgHdr.messageId;
  var folderURI = aMsgHdr.folder.URI;
  
  for (var i = 0; i < aMsgHdrArray.Count(); i++)
  {
    var msgHdr = aMsgHdrArray.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgDBHdr);
    if (msgHdr.messageId == messageId && msgHdr.folder.URI == folderURI)
      return false;
  }
  return true;
}

function ToggleMessageTag(key, addKey)
{
  var selectedMsgUris = GetSelectedMessages();
  ToggleMessageTagInternal(key, addKey, selectedMsgUris);
  NotifyTagChanged();
}

function SetMessageTagLabel(menuitem, index, name)
{
  // if a <key> is defined for this tag, use its key as the accesskey
  // (the key for the tag at index n needs to have the id key_tag<n>)
  var shortcutkey = document.getElementById("key_tag" + index);
  var accesskey = shortcutkey ? shortcutkey.getAttribute("key") : "";
  if (accesskey)
    menuitem.setAttribute("accesskey", accesskey);
  var label = gMessengerBundle.getFormattedString("mailnews.tags.format", 
                                                  [accesskey, name]);
  menuitem.setAttribute("label", label);
}

function InitMessageTags(menuPopup, aAddNewTagId, aRemoveAllTagId)
{
  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                             .getService(Components.interfaces.nsIMsgTagService);
  var tagArray = tagService.getAllTags({});
  var tagCount = tagArray.length;

  var addTagMenuItem = document.getElementById(aAddNewTagId);
  // remove any add tag values
  while (addTagMenuItem.nextSibling.localName != 'menuseparator')
    menuPopup.removeChild(addTagMenuItem.nextSibling);
  var menuSeparator = addTagMenuItem.nextSibling;

  while (menuPopup.lastChild.id != aRemoveAllTagId)
    menuPopup.removeChild(menuPopup.lastChild);

  // now rebuild the list
  var msgHdr = gDBView.hdrForFirstSelectedMessage;
  var curKeys = msgHdr.getStringProperty("keywords");
  if (msgHdr.label)
    curKeys += " $label" + msgHdr.label;

  var activeTagCount = 1;
  for (var i = 0; i < tagCount; ++i)
  {
    var taginfo = tagArray[i];
    if (taginfo.active)
    {
      // TODO we want to either remove or "check" the tags that already exist
      var newMenuItem = document.createElement("menuitem");
      SetMessageTagLabel(newMenuItem, activeTagCount++, taginfo.tag);
      newMenuItem.setAttribute("value", taginfo.key);
      newMenuItem.setAttribute('oncommand', 'ToggleMessageTagMenu(event.target);');
      newMenuItem.className = 'tagMenuItem';
      var removeKey = (" " + curKeys + " ").indexOf(" " + taginfo.key + " ") > -1;
      if (removeKey)
        menuPopup.appendChild(newMenuItem);
      else
      {
        newMenuItem.setAttribute('addkey', true); // used to aid in toggling
        menuPopup.insertBefore(newMenuItem, menuSeparator);
      }
    }
  }
}

function backToolbarMenu_init(menuPopup)
{
  populateHistoryMenu(menuPopup, true);
}

var gNavDebug = false;
function navDebug(str)
{
  if (gNavDebug)
    dump(str);
}

function populateHistoryMenu(menuPopup, isBackMenu)
{
  // remove existing entries
  while (menuPopup.firstChild)
    menuPopup.removeChild(menuPopup.firstChild);
  var curPos = new Object;
  var numEntries = new Object;
  var historyEntries = new Object;
  messenger.getNavigateHistory(curPos, numEntries, historyEntries);
  curPos.value = curPos.value * 2;
  navDebug("curPos = " + curPos.value + " numEntries = " + numEntries.value + "\n");
  var historyArray = historyEntries.value;
  var folder;
  var newMenuItem;
  if (GetLoadedMessage())
  {
    if (!isBackMenu)
      curPos.value += 2;
    else
      curPos.value -= 2;
  }
  // for populating the back menu, we want the most recently visited
  // messages first in the menu. So we go backward from curPos to 0.
  // For the forward menu, we want to go forward from curPos to the end.
  
  var relPos = 0;
  for (var i = curPos.value; (isBackMenu) ? i >= 0 : i < historyArray.length; i += ((isBackMenu) ? -2 : 2))
  {
    navDebug("history[" + i + "] = " + historyArray[i] + "\n");
    navDebug("history[" + i + "] = " + historyArray[i + 1] + "\n");
    folder = RDF.GetResource(historyArray[i + 1]).QueryInterface(Components.interfaces.nsIMsgFolder);
    navDebug("folder URI = " + folder.URI + "pretty name " + folder.prettyName + "\n");
    var menuText = "";
    
    var msgHdr = messenger.msgHdrFromURI(historyArray[i]);
    if (!IsCurrentLoadedFolder(folder))
      menuText = folder.prettyName + " - ";

    var subject = "";
    if (msgHdr.flags & MSG_FLAG_HAS_RE)
      subject = "Re: ";
    if (msgHdr.mime2DecodedSubject)
      subject += msgHdr.mime2DecodedSubject;
    if (subject)
      menuText += subject + " - ";

    menuText += msgHdr.mime2DecodedAuthor;
    newMenuItem = document.createElement('menuitem');
    newMenuItem.setAttribute('label', menuText);
    relPos += isBackMenu ? -1 : 1;
    newMenuItem.setAttribute('value',  relPos);
    newMenuItem.setAttribute('oncommand', 'NavigateToUri(event.target); event.stopPropagation();');
    menuPopup.appendChild(newMenuItem);
    if (! (relPos % 20))
      break;
  }
  
}

function NavigateToUri(target)
{
  var historyIndex = target.getAttribute('value');
  var folderUri = messenger.getFolderUriAtNavigatePos(historyIndex);
  var msgUri = messenger.getMsgUriAtNavigatePos(historyIndex);
  var folder = RDF.GetResource(folderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
  var msgHdr = messenger.msgHdrFromURI(msgUri);
  navDebug("navigating from " + messenger.navigatePos + " by " + historyIndex + " to " + msgUri + "\n");
  navDebug("folderUri = " + folderUri + "\n");
  // this "- 0" seems to ensure that historyIndex is treated as an int, not a string.
  messenger.navigatePos += (historyIndex - 0);
  LoadNavigatedToMessage(msgHdr, folder, folderUri);
}

function forwardToolbarMenu_init(menuPopup)
{
  populateHistoryMenu(menuPopup, false);
}

/*
 * @param aMsgHdr optional parameter
 */
function InitMessageMark(aPopup, aMsgHdr)
{
  if (!aPopup)
    return;

  var areMessagesRead;
  if (aMsgHdr == undefined)
    areMessagesRead = SelectedMessagesAreRead();
  else
    areMessagesRead = aMsgHdr.isRead;

  aPopup.getElementsByAttribute('action', 'read')[0].hidden = areMessagesRead;
  aPopup.getElementsByAttribute('action', 'unread')[0].hidden = !areMessagesRead;

  var areMessagesPending;
  if (aMsgHdr == undefined)
    areMessagesPending = SelectedMessagesArePending();
  else
    areMessagesPending = aMsgHdr.flags & MSG_FLAG_PENDING;
  
  aPopup.getElementsByAttribute('action', 'pending')[0].hidden = areMessagesPending;
  aPopup.getElementsByAttribute('action', 'notpending')[0].hidden = !areMessagesPending;

  var areMessagesFlagged;
  if (aMsgHdr == undefined)
    areMessagesFlagged = SelectedMessagesAreFlagged();
  else
    areMessagesFlagged = aMsgHdr.isFlagged;

  aPopup.getElementsByAttribute('action', 'flagged')[0].hidden= areMessagesFlagged;
  aPopup.getElementsByAttribute('action', 'notflagged')[0].hidden = !areMessagesFlagged;

  var areMessagesJunk;
  if (aMsgHdr == undefined)
    areMessagesJunk = SelectedMessagesAreJunk();
  else
  {
    var junkScore = aMsgHdr.getStringProperty("junkscore");
    areMessagesJunk =  ((junkScore != "") && (junkScore != "0")) || (aMsgHdr.folder.flags & MSG_FOLDER_FLAG_JUNK);
  }

  aPopup.getElementsByAttribute('action', 'junk')[0].hidden = areMessagesJunk;
  aPopup.getElementsByAttribute('action', 'notjunk')[0].hidden = !areMessagesJunk;

  document.commandDispatcher.updateCommands('create-menu-mark');
}

function UpdateJunkToolbarButton()
{
  var junkButtonDeck = document.getElementById("junk-deck");
  if (junkButtonDeck)
    junkButtonDeck.selectedIndex = SelectedMessagesAreJunk() ? 1 : 0;
}

function UpdateDeleteCommand()
{
  var value = "value";
  var uri = GetFirstSelectedMessage();
  if (IsNewsMessage(uri))
    value += "News";
  else if (SelectedMessagesAreDeleted())
    value += "IMAPDeleted";
  if (GetNumSelectedMessages() < 2)
    value += "Message";
  else
    value += "Messages";
  goSetMenuValue("cmd_delete", value);
  goSetAccessKey("cmd_delete", value + "AccessKey");
}

function SelectedMessagesAreDeleted()
{
    try {
        return gDBView.hdrForFirstSelectedMessage.flags & MSG_FLAG_IMAP_DELETED;
    }
    catch (ex) {
        return 0;
    }
}

function SelectedMessagesAreJunk()
{
    var isJunk;
    try {
        var junkScore = gDBView.hdrForFirstSelectedMessage.getStringProperty("junkscore");
        isJunk =  ((junkScore != "") && (junkScore != "0")) || (gDBView.hdrForFirstSelectedMessage.folder.flags & MSG_FOLDER_FLAG_JUNK);
    }
    catch (ex) {
        isJunk = false;
    }
    return isJunk;
}

function SelectedMessagesAreRead()
{
    var isRead;
    try {
        isRead = gDBView.hdrForFirstSelectedMessage.isRead;
    }
    catch (ex) {
        isRead = false;
    }
    return isRead;
}

function SelectedMessagesAreFlagged()
{
    var isFlagged;
    try {
        isFlagged = gDBView.hdrForFirstSelectedMessage.isFlagged;
    }
    catch (ex) {
        isFlagged = false;
    }
    return isFlagged;
}

function SelectedMessagesArePending()
{
  var isPending;
  try {
    isPending = gDBView.hdrForFirstSelectedMessage.flags & MSG_FLAG_PENDING;
  }
  catch (ex) {
    isPending = false;
  }
  return isPending;
}

function getMsgToolbarMenu_init()
{
    document.commandDispatcher.updateCommands('create-menu-getMsgToolbar');
}

function GetFirstSelectedMsgFolder()
{
    var result = null;
    var selectedFolders = GetSelectedMsgFolders();
    if (selectedFolders.length > 0) {
        result = selectedFolders[0];
    }

    return result;
}

function GetWindowMediator()
{
    if (gWindowManagerInterface)
        return gWindowManagerInterface;

    var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
    return (gWindowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator));
}

function GetInboxFolder(server)
{
    try {
        var rootMsgFolder = server.rootMsgFolder;

        //now find Inbox
        var outNumFolders = new Object();
        var inboxFolder = rootMsgFolder.getFoldersWithFlag(0x1000, 1, outNumFolders);
        
        if (outNumFolders.value)
          return inboxFolder.QueryInterface(Components.interfaces.nsIMsgFolder);
        else
          return null;
    }
    catch (ex) {
        dump(ex + "\n");
    }
    return null;
}

function GetMessagesForInboxOnServer(server)
{
  var inboxFolder = GetInboxFolder(server);

  // if the server doesn't support an inbox it could be an RSS server or some other server type..
  // just use the root folder and the server implementation can figure out what to do.
  if (!inboxFolder)
    inboxFolder = server.rootFolder;    

  var folders = new Array(1);
  folders[0] = inboxFolder;
  GetNewMessages(folders, server);
}

function MsgGetMessage()
{
  // if offline, prompt for getting messages
  if (MailOfflineMgr.isOnline() || MailOfflineMgr.getNewMail())
    GetFolderMessages();
}

function MsgGetMessagesForAllServers(defaultServer)
{
    // now log into any server
    try
    {
        var allServers = accountManager.allServers;
        var i;
        // array of isupportsarrays of servers for a particular folder
        var pop3DownloadServersArray = new Array();
        // parallel isupports array of folders to download to...
        var localFoldersToDownloadTo = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
        var pop3Server;

        for (i = 0; i < allServers.Count(); ++i)
        {
            var currentServer = allServers.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
            var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + currentServer.type].getService(Components.interfaces.nsIMsgProtocolInfo);
            if (protocolinfo.canLoginAtStartUp && currentServer.loginAtStartUp)
            {
                if (defaultServer && defaultServer.equals(currentServer) && 
                  !defaultServer.isDeferredTo &&
                  defaultServer.rootFolder == defaultServer.rootMsgFolder)
                {
                    dump(currentServer.serverURI + "...skipping, already opened\n");
                }
                else if (currentServer.type == "pop3" && currentServer.downloadOnBiff)
                {
                    CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
                    pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
                }
                else
                {
                    // Check to see if there are new messages on the server
                    currentServer.performBiff(msgWindow);
                }
            }
        }
        for (i = 0; i < pop3DownloadServersArray.length; ++i)
        {
          // any ol' pop3Server will do - the serversArray specifies which servers to download from
          pop3Server.downloadMailFromServers(pop3DownloadServersArray[i], msgWindow, localFoldersToDownloadTo.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgFolder), null);
        }
    }
    catch(ex)
    {
        dump(ex + "\n");
    }
}

/**
  * Get messages for all those accounts which have the capability
  * of getting messages and have session password available i.e.,
  * curretnly logged in accounts.
  * if offline, prompt for getting messages.
  */
function MsgGetMessagesForAllAuthenticatedAccounts()
{
  if (MailOfflineMgr.isOnline() || MailOfflineMgr.getNewMail())
    GetMessagesForAllAuthenticatedAccounts();
}

/**
  * Get messages for the account selected from Menu dropdowns.
  * if offline, prompt for getting messages.
  */
function MsgGetMessagesForAccount(aEvent)
{
  if (!aEvent)
    return;

  if (MailOfflineMgr.isOnline() || MailOfflineMgr.getNewMail())
    GetMessagesForAccount(aEvent);
}

// if offline, prompt for getNextNMessages
function MsgGetNextNMessages()
{
  var folder;
  if (MailOfflineMgr.isOnline() || MailOfflineMgr.getNewMail()) 
  {
    folder = GetFirstSelectedMsgFolder();
    if (folder) 
      GetNextNMessages(folder);
  }
}   

function MsgDeleteMessage(reallyDelete, fromToolbar)
{
    // if from the toolbar, return right away if this is a news message
    // only allow cancel from the menu:  "Edit | Cancel / Delete Message"
    if (fromToolbar)
    {
        var srcFolder = GetLoadedMsgFolder();
        var folderResource = srcFolder.QueryInterface(Components.interfaces.nsIRDFResource);
        var uri = folderResource.Value;
        if (isNewsURI(uri)) {
            // if news, don't delete
            return;
        }
    }

    SetNextMessageAfterDelete();
    if (reallyDelete) {
        gDBView.doCommand(nsMsgViewCommandType.deleteNoTrash);
    }
    else {
        gDBView.doCommand(nsMsgViewCommandType.deleteMsg);
    }
}

// MsgCopyMessage
//   Copies the selected messages to the destination folder
//   aDestFolderURI -- the URI of the destination folder
function MsgCopyMessage(aDestFolderURI)
{
  try {
    // get the msg folder we're copying messages into
    var destResource = RDF.GetResource(aDestFolderURI);
    var destMsgFolder = destResource.QueryInterface(Components.interfaces.nsIMsgFolder);
    gDBView.doCommandWithFolder(nsMsgViewCommandType.copyMessages, destMsgFolder);
    pref.setCharPref("mail.last_msg_movecopy_target_uri", aDestFolderURI);
    pref.setBoolPref("mail.last_msg_movecopy_was_move", false);  
  }
  catch (ex) {
    dump("MsgCopyMessage failed: " + ex + "\n");
  }
}

function MsgArchiveMessage()
{
  gDBView.doCommand(nsMsgViewCommandType.archive);
}

// MsgMoveMessage
//   Moves the selected messages to the destination folder
//   aDestFolderURI -- the URI of the destination folder
function MsgMoveMessage(aDestFolderURI)
{
  try {
    // get the msg folder we're moving messages into
    // var destUri = destFolder.getAttribute('id');
    var destResource = RDF.GetResource(aDestFolderURI);
    var destMsgFolder = destResource.QueryInterface(Components.interfaces.nsIMsgFolder);
    // we don't move news messages, we copy them
    if (isNewsURI(gDBView.msgFolder.URI))
      gDBView.doCommandWithFolder(nsMsgViewCommandType.copyMessages, destMsgFolder);
    else 
    {
      SetNextMessageAfterDelete();
      gDBView.doCommandWithFolder(nsMsgViewCommandType.moveMessages, destMsgFolder);
    }    
    pref.setCharPref("mail.last_msg_movecopy_target_uri", aDestFolderURI);
    pref.setBoolPref("mail.last_msg_movecopy_was_move", true);  
  }
  catch (ex) {
    dump("MsgMoveMessage failed: " + ex + "\n");
  }
}

function MsgNewMessage(event)
{
  ComposeMessage(msgComposeType.New, event && event.shiftKey ? msgComposeFormat.OppositeOfDefault : msgComposeFormat.Default,
                 GetFirstSelectedMsgFolder(), "");
}

/**
 * @param aComposeType ReplyToSender, ReplyToGroup, ReplyAll
 * @param aUseDefaultComposeFormat use the default compose format
 * @param aForceSender - hacky...don't replace the format type with reply to group if we have a chance to.
 *                        used to support reply to sender for newsgroups without requiring a separate method
 */
function MsgReply(aUseDefaultComposeFormat, aReplyAll, aForceSender)
{
  var loadedFolder = GetLoadedMsgFolder();
  var messageArray = getSelectedConversationMessages() || GetSelectedMessages();

  var composeType = aReplyAll ? msgComposeType.ReplyAll : msgComposeType.ReplyToSender;
  if (!aForceSender && !aReplyAll && loadedFolder && loadedFolder.server && loadedFolder.server.type == 'nntp')
    composeType = msgComposeType.ReplyToGroup;

  ComposeMessage(composeType, aUseDefaultComposeFormat ? msgComposeFormat.Default : msgComposeFormat.OppositeOfDefault,
                 loadedFolder, messageArray, "");
}

function GetForwardType()
{
  // mail.forward_message_mode could be 1, if the user migrated from 4.x
  // 1 (forward as quoted) is obsolete, so we treat is as forward inline
  // since that is more like forward as quoted then forward as attachment
  var forwardType = 0;
  try {
    forwardType = gPrefBranch.getIntPref("mail.forward_message_mode");
  }
  catch (ex) {
    dump("failed to retrieve pref mail.forward_message_mode");
  }
  return forwardType;
}

function MsgForwardMessage(event)
{
  if (GetForwardType() == 0)
      MsgForwardAsAttachment(event);
  else
      MsgForwardAsInline(event);
}

function MsgForwardAsAttachment(event)
{
  var loadedFolder = GetLoadedMsgFolder();
  var messageArray = getSelectedConversationMessages() || GetSelectedMessages();

  //dump("\nMsgForwardAsAttachment from XUL\n");
  if (event && event.shiftKey)
    ComposeMessage(msgComposeType.ForwardAsAttachment,
                   msgComposeFormat.OppositeOfDefault, loadedFolder, messageArray, "");
  else
    ComposeMessage(msgComposeType.ForwardAsAttachment, msgComposeFormat.Default, loadedFolder, messageArray, "");
}

function MsgForwardAsInline(event)
{
  var loadedFolder = GetLoadedMsgFolder();
  var messageArray = getSelectedConversationMessages() || GetSelectedMessages();

  //dump("\nMsgForwardAsInline from XUL\n");
  if (event && event.shiftKey)
    ComposeMessage(msgComposeType.ForwardInline,
                   msgComposeFormat.OppositeOfDefault, loadedFolder, messageArray, "");
  else
    ComposeMessage(msgComposeType.ForwardInline, msgComposeFormat.Default, loadedFolder, messageArray, "");
}

/**
 * if we are in a draft folder, edit message --> open the draft
 * otherwise edit the message
 */
function MsgEditMessage()
{
  var loadedFolder = GetLoadedMsgFolder();
  var messageArray = GetSelectedMessages();
  var isDraft = IsSpecialFolderSelected(MSG_FOLDER_FLAG_DRAFTS, false);
  if (!isDraft)
    ReportAnalytics("edit:message");
  ComposeMessage(isDraft ? msgComposeType.Draft : msgComposeType.Edit, msgComposeFormat.Default, loadedFolder, messageArray, "");
}

function MsgEditMessageAsNew()
{
    var loadedFolder = GetLoadedMsgFolder();
    var messageArray = getSelectedConversationMessages() || GetSelectedMessages();
    ComposeMessage(msgComposeType.Template, msgComposeFormat.Default, loadedFolder, messageArray, "");
}

function MsgRedirect()
{
  var loadedFolder = GetLoadedMsgFolder();
  var messageArray = getSelectedConversationMessages() || GetSelectedMessages();
  ComposeMessage(msgComposeType.Redirect, msgComposeFormat.Default, loadedFolder, messageArray, "");
}

function MsgCreateFilter()
{
  // retrieve Sender direct from selected message's headers
  var msgHdr = gDBView.hdrForFirstSelectedMessage;
  var headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
  var emailAddress = headerParser.extractHeaderAddressMailboxes(null, msgHdr.author);
  if (emailAddress)
    top.MsgFilters(emailAddress, null);
}

function MsgHome(url)
{
  window.open(url, "_blank", "chrome,dependent=yes,all");
}

/**
 *
 * @param preselectedFolder - optional argument will use the folder selection if empty
 */
function MsgNewFolder(callBackFunctionName, preselectedFolder)
{
    if (!preselectedFolder)
      preselectedFolder = GetFirstSelectedMsgFolder();
    var dualUseFolders = true;
    var server = null;
    var destinationFolder = null;

    if (preselectedFolder)
    {
      // use the first real folder in the smart folder
      if (SmartServerUtils.isSmartFolder(preselectedFolder))
         preselectedFolder = SmartServerUtils.foldersForSmartFolder(preselectedFolder)[0];
        try {
            server = preselectedFolder.server;
            if (server)
            {
                destinationFolder = getDestinationFolder(preselectedFolder, server);

                var imapServer =
                    server.QueryInterface(Components.interfaces.nsIImapIncomingServer);
                if (imapServer)
                    dualUseFolders = imapServer.dualUseFolders;
            }
        } catch (e) {
            dump ("Exception: dualUseFolders = true\n");
        }
    }
    CreateNewSubfolder("chrome://messenger/content/newFolderDialog.xul", destinationFolder, dualUseFolders, callBackFunctionName);
}

function getDestinationFolder(preselectedFolder, server)
{
    var destinationFolder = null;

    var isCreateSubfolders = preselectedFolder.canCreateSubfolders;
    if (!isCreateSubfolders)
    {
        destinationFolder = server.rootMsgFolder;

        var verifyCreateSubfolders = null;
        if (destinationFolder)
            verifyCreateSubfolders = destinationFolder.canCreateSubfolders;

        // in case the server cannot have subfolders,
        // get default account and set its incoming server as parent folder
        if (!verifyCreateSubfolders)
        {
            try {
                var defaultFolder = GetDefaultAccountRootFolder();
                var checkCreateSubfolders = null;
                if (defaultFolder)
                    checkCreateSubfolders = defaultFolder.canCreateSubfolders;

                if (checkCreateSubfolders)
                    destinationFolder = defaultFolder;

            } catch (e) {
                dump ("Exception: defaultAccount Not Available\n");
            }
        }
    }
    else
        destinationFolder = preselectedFolder;

    return destinationFolder;
}

/** Open subscribe window. */
function MsgSubscribe()
{
  var preselectedFolder = GetFirstSelectedMsgFolder();

  var server = (preselectedFolder) ? preselectedFolder.server : null;
  if (server && server.type == "rss")
    openSubscriptionsDialog(server); // open feed subscription dialog
  else
    Subscribe(preselectedFolder); // open imap/nntp subscription dialog
}

function ConfirmUnsubscribe(folder)
{
  if (!gMessengerBundle)
    gMessengerBundle = document.getElementById("bundle_messenger");

  var titleMsg = gMessengerBundle.getString("confirmUnsubscribeTitle");
  var dialogMsg = gMessengerBundle.getFormattedString("confirmUnsubscribeText",
                                      [folder.name], 1);

  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
  return promptService.confirm(window, titleMsg, dialogMsg);
}

function MsgUnsubscribe()
{
  var folder = GetFirstSelectedMsgFolder();
  if (ConfirmUnsubscribe(folder)) {
    UnSubscribe(folder);
  }
}

function ToggleFavoriteFolderFlag()
{
  var folder = GetFirstSelectedMsgFolder();
  folder.toggleFlag(MSG_FOLDER_FLAG_FAVORITE);
  // rebuild the favorites bar
  gFavoritesBar.rebuild();
}

function MsgSaveAsFile()
{
  SaveAsFile(getSelectedConversationMessages() || GetSelectedMessages());
}

function MsgSaveAsTemplate()
{
  var folder = GetLoadedMsgFolder();
  if (GetNumSelectedMessages() == 1)
    SaveAsTemplate(GetFirstSelectedMessage(), folder);
}

function MsgOpenNewWindowForMsgHdr(hdr)
{
  MsgOpenNewWindowForFolder(hdr.folder.URI, hdr.messageKey);
}

function MsgOpenNewWindowForFolder(uri, key)
{
  var uriToOpen = uri;
  var keyToSelect = key;

  if (!uriToOpen)
    // use GetSelectedFolderURI() to find out which message to open instead of
    // GetLoadedMsgFolder().QueryIntervace(Components.interfaces.nsIRDFResource).value.
    // This is required because on a right-click, the currentIndex value will be
    // different from the actual row that is highlighted.  GetSelectedFolderURI()
    // will return the message that is highlighted.
    uriToOpen = GetSelectedFolderURI();

  if (uriToOpen)
    window.openDialog("chrome://messenger/content/", "_blank", "chrome,all,dialog=no", uriToOpen, keyToSelect);
}

function CreateToolbarTooltip(document, event)
{
  event.stopPropagation();
  var tn = document.tooltipNode;
  if (tn.localName != "tab")
    return false; // Not a tab, so cancel the tooltip
  if ("mOverCloseButton" in tn && tn.mOverCloseButton) {
     event.target.setAttribute("label", tn.getAttribute("closetabtext"));
     return true;
  }
  if (tn.hasAttribute("label")) {
    event.target.setAttribute("label", tn.getAttribute("label"));
    return true;
  }
  return false;
}

/**
 * @param aBackgroundTab true if the tab should be opened in the background
 */
function MsgOpenNewTabForFolderPaneSelection(aBackgroundTab)
{
  MsgOpenNewTabForFolderWithAnimation(null, -1, aBackgroundTab, false);
  PersistTabs();
}

/**
 * This is a legacy routine used for opening a new tab from session restore.
 * Most callers should be using MsgOpenNewTabForFolderWithAnimation.
 */
function MsgOpenNewTabForFolder(uriToOpen, key, aBackground, aAccountURI)
{
  return MsgOpenNewTabForFolderWithAnimation(uriToOpen, key, aBackground, true /* skip when opening a restored tab */, aAccountURI);
}

/**
 * will avoid opening the tab if there is an error turning uriToOpen into a nsIMsgFolder or
 * if the folder does not have an incoming server (because the account is no longer valid. 
 * This avoids creating bad message tabs at start up.
 * @param uri
 * @param key
 * @param aBackground open the tab in the background
 * @param aAccountURI (optional) the preferred account to select in the account tree
 */
function MsgOpenNewTabForFolderWithAnimation(uriToOpen, key, aBackground, aSkipAnimation, aAccountURI)
{  
  if (!uriToOpen)
  {
    // use GetSelectedFolderURI() to find out which message to open instead of
    // GetLoadedMsgFolder().QueryIntervace(Components.interfaces.nsIRDFResource).value.
    // This is required because on a right-click, the currentIndex value will be
    // different from the actual row that is highlighted.  GetSelectedFolderURI()
    // will return the message that is highlighted.
    uriToOpen = GetSelectedFolderURI();
    // hack - avoid opening a new tab on all mail folders...just revert to the underlying folder
    if (uriToOpen == kAllMailFoldersURI)
      uriToOpen = gDBView.msgFolder.URI;
    aAccountURI = GetSelectedAccountURI();
  }

  let folder = GetMsgFolderFromUri(uriToOpen);
  let server;
  try {
    server = folder.server;
  }
  catch (ex)
  {
    Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage("MsgOpenNewTabForFolder failed, uriToOpen = " + uriToOpen + " ex = " + ex);
    // folder.server throws an error if the account no longer exists
    // use the inbox of the default account, if we can find it.
    // see bug #5722 for details
    try
    {
       var defaultAccount = accountManager.defaultAccount;
       server = defaultAccount.incomingServer;
       var rootMsgFolder = server.rootMsgFolder;

       // now find Inbox
       var outNumFolders = new Object();
       folder = rootMsgFolder.getFoldersWithFlag(0x1000, 1, outNumFolders);
       if (folder)
       {
         uriToOpen = folder.URI;
         Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage("MsgOpenNewTabForFolder override uriToOpen = " + uriToOpen);
       }
    }
    catch (ex)
    {
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage("MsgOpenNewTabForFolder failed to find inbox on default account, ex = " + ex);
    }
  }

  if (folder && server)
  {
    let newTab = new folderTabOwner(uriToOpen, key);
    document.getElementById('tabmail').addTab(newTab, {background: aBackground, skipAnimation: aSkipAnimation, accountURI: aAccountURI});
  }
}

/**
 * Opens a new message tab for the selected message in the thread pane.
 * uses the first selected message in the view to call MsgOpenNewTabForMessage
 */
function MsgOpenNewTabForSelectedMessage(aBackground)
{
  var msgHdr = gDBView.hdrForFirstSelectedMessage;
  // if we are showing a conversation, pass that information in
  // XXX TO DO - use the view to figure out if the dummy row is selected
  MsgOpenNewTabForMessage(msgHdr.messageKey, msgHdr.folder.URI,
                          messageHeaderSink._conversationMode ? 'conversation' : '',
                          aBackground, false, GetSelectedAccountURI());
}

/**
 * This is a legacy method used for tab session restore. Most callers will want to use
 * MsgOpenNewTabForMessageWithAnimation.
 */
function MsgOpenNewTabForMessage(aMessageKey, aFolderURI, aMode /* conversation */, aBackground, aAccountURI)
{
  return MsgOpenNewTabForMessageWithAnimation(aMessageKey, aFolderURI, aMode, aBackground, true /* skip animation */, aAccountURI);
}

/**
 * open a new message tab based on aMessageKey and aFolderURI
 * will avoid opening the tab if there is an error turning aFolderURI into a nsIMsgFolder or
 * if aMessageKey is no longer in the folder. This avoids creating bad message tabs at start up.
 *
 * @param aMessageKey
 * @param aFolderURI
 * @param aMode - 'reconstructed-conversation' or 'conversation'. empty string to just open a normal message
 * @param aBackground true if the tab should be opened in the background
 * @param aSkipAnimation 
 * @param aAccountURI (optional) the preferred account to select in the account tree
 */
function MsgOpenNewTabForMessageWithAnimation(aMessageKey, aFolderURI, aMode /* conversation */, aBackground, aSkipAnimation, aAccountURI)
{
  let folder = RDF.GetResource(aFolderURI);
  if (folder && folder instanceof Components.interfaces.nsIMsgFolder)
  {
    // make sure the server is still valid
    let server;
    // folder.server throws an error if the account no longer exists
    try {
      server = folder.server;
    } catch(ex) {}
    
    if (server)
    {
      var hdr = folder.GetMessageHeader(aMessageKey);
      // if we can't find the msgHdr for aMessageKey, then don't bother trying to open the tab, 
      // since that will just end up creating a bogus tab.
      var db = folder.getMsgDatabase(msgWindow);
      if (hdr && db && db.ContainsKey(aMessageKey))
      {
        // fix it so we won't try to load the previously loaded message.
        folder.lastMessageLoaded = nsMsgKey_None;

        var tab = new messageTabOwner(hdr, aMode);
        document.getElementById('tabmail').addTab(tab, {background: aBackground, skipAnimation: aSkipAnimation, accountURI: aAccountURI});
        PersistTabs();
      }
    }
  }
}

// passing in the view, so this will work for search and the thread pane
function MsgOpenSelectedMessages()
{
  var dbView = GetDBView();

  var indices = GetSelectedIndices(dbView);
  var numMessages = indices.length;
  var openMsgBehavior = gPrefBranch.getIntPref("mail.tabs.open_message");

  if (openMsgBehavior == 2 && numMessages == 1 && MsgOpenSelectedMessageInExistingWindow())
    return;

  var openWindowWarning = gPrefBranch.getIntPref("mailnews.open_window_warning");
  if ((openWindowWarning > 1) && (numMessages >= openWindowWarning)) {
    if (!gMessengerBundle)
        gMessengerBundle = document.getElementById("bundle_messenger");
    var title = gMessengerBundle.getString("openWindowWarningTitle");
    var text = gMessengerBundle.getFormattedString("openWindowWarningText", [numMessages]);
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                          .getService(Components.interfaces.nsIPromptService);    
    if (!promptService.confirm(window, title, text))
      return;
  }

  let selConversationMsgs = getSelectedConversationMessages();
  if (selConversationMsgs)
  {
    let msgHdr = messenger.msgHdrFromURI(selConversationMsgs[0]);
    if (openMsgBehavior != 0 || document.documentElement.getAttribute('windowtype') != "mail:3pane") // only the 3 pane supports opening in a tab
      MsgOpenNewWindowForMessage(selConversationMsgs[0], msgHdr.folder.URI);
    else
    {
      MsgOpenNewTabForMessageWithAnimation(msgHdr.messageKey, msgHdr.folder.URI, 
                                           false, false, false, GetSelectedAccountURI());
      return;
    }
  }
  else
  {
    for (var i = 0; i < numMessages; i++) 
    {
      if (openMsgBehavior != 0 || document.documentElement.getAttribute('windowtype') != "mail:3pane") // only the 3 pane supports opening in a tab
        MsgOpenNewWindowForMessage(dbView.getURIForViewIndex(indices[i]), dbView.getFolderForViewIndex(indices[i]).URI);
      else
      {
        MsgOpenNewTabForMessageWithAnimation(dbView.getKeyAt(indices[i]), dbView.getFolderForViewIndex(indices[i]).URI, 
                                messageHeaderSink._conversationMode ? 'conversation' : '', false, numMessages > 1, GetSelectedAccountURI());
        return;
      }
    }
  }
}

function MsgOpenSelectedMessageInExistingWindow()
{
  var windowID = GetWindowByWindowType("mail:messageWindow");
  if (!windowID)
    return false;

  try {
    var messageURI = gDBView.URIForFirstSelectedMessage;
    var msgHdr = gDBView.hdrForFirstSelectedMessage;

    // Reset the window's message uri and folder uri vars, and
    // update the command handlers to what's going to be used.
    // This has to be done before the call to CreateView().
    windowID.gCurrentMessageUri = messageURI;
    windowID.gCurrentFolderUri = msgHdr.folder.URI;
    windowID.UpdateMailToolbar('MsgOpenExistingWindowForMessage');

    // even if the folder uri's match, we can't use the existing view
    // (msgHdr.folder.URI == windowID.gCurrentFolderUri)
    // the reason is quick search and mail views.
    // see bug #187673
    //
    // for the sake of simplicity,
    // let's always call CreateView(gDBView)
    // which will clone gDBView
    windowID.CreateView(gDBView);
    windowID.LoadMessageByMsgKey(msgHdr.messageKey);

    // bring existing window to front
    windowID.focus();
    return true;
  }
  catch (ex) {
    dump("reusing existing standalone message window failed: " + ex + "\n");
  }
  return false;
}

function MsgOpenFromFile()
{
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

  var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService();
  strBundleService = strBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
  var extbundle = strBundleService.createBundle("chrome://messenger/locale/messenger.properties");
  var filterLabel = extbundle.GetStringFromName("EMLFiles");
  var windowTitle = extbundle.GetStringFromName("OpenEMLFiles");

  fp.init(window, windowTitle, nsIFilePicker.modeOpen);
  fp.appendFilter(filterLabel, "*.eml");

  // Default or last filter is "All Files"
  fp.appendFilters(nsIFilePicker.filterAll);

  try {
    var ret = fp.show();
    if (ret == nsIFilePicker.returnCancel)
      return;
  }
  catch (ex) {
    dump("filePicker.chooseInputFile threw an exception\n");
    return;
  }

  var uri = fp.fileURL.QueryInterface(Components.interfaces.nsIURL); 
  uri.query = "type=application/x-message-display";

  window.openDialog("chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", uri, null, null );
}

function MsgOpenNewWindowForMessage(messageUri, folderUri, aSkipView)
{
    if (!messageUri)
        // use GetFirstSelectedMessage() to find out which message to open
        // instead of gDBView.getURIForViewIndex(currentIndex).  This is
        // required because on a right-click, the currentIndex value will be
        // different from the actual row that is highlighted.
        // GetFirstSelectedMessage() will return the message that is
        // highlighted.
        messageUri = GetFirstSelectedMessage();

    if (!folderUri)
        // use GetSelectedFolderURI() to find out which message to open
        // instead of gDBView.getURIForViewIndex(currentIndex).  This is
        // required because on a right-click, the currentIndex value will be
        // different from the actual row that is highlighted.
        // GetSelectedFolderURI() will return the message that is
        // highlighted.
        folderUri = GetSelectedFolderURI();
    try {
      if (folderUri == kAllMailFoldersURI)
      {
        // don't pass in all mail folders as a folder uri...find the folder for the message we want to display
        folderUri = messenger.msgHdrFromURI(messageUri).folder.URI;
      }
    }
    catch (ex) {
      var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
      consoleService.logStringMessage('exception in MsgOpenNewWindowForMessage: ' + ex);
    }

    // be sure to pass in the current view....
    if (messageUri && folderUri) {
        window.openDialog( "chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", messageUri, folderUri, aSkipView ? null : gDBView );
    }
}

function CloseMailWindow()
{
  // XXX todo - when we figure out a generic way to abort most of the commands when editing an inline message,
  // we should remove this check. But for now, we don't know how we're going to tackle that problem yet.
  if (!gPbActionMonitor.confirmAction())
    return;
  var tabmail = document.getElementById('tabmail');
  if (!tabmail || tabmail.mTabContainer.childNodes.length == 1)
  {
    // don't close the window if we are in the middle of a send
    if (!("pbConfirmOkToQuitWhileSending" in window) || !pbConfirmOkToQuitWhileSending())
      window.close();
  }
  else if (!tabmail.mTabContainer.selectedItem.hasAttribute('noclose'))
    tabmail.removeTab(tabmail.mTabContainer.selectedItem, {skipAnimation: false});
}

function MsgJunk()
{
  MsgJunkMailInfo(true);
  JunkSelectedMessages(!SelectedMessagesAreJunk());
}

/**
 * Checks if the selected messages can be marked as read or unread
 *
 * @param read true if trying to mark messages as read, false otherwise
 * @return true if the chosen operation can be performed
 */
function CanMarkMsgAsRead(read)
{
  return SelectedMessagesAreRead() != read;
}

/**
 * Marks the selected messages as read or unread
 *
 * @param read true if trying to mark messages as read, false if marking unread,
 *        undefined if toggling the read status
 */
function MsgMarkMsgAsRead(read)
{
  if (read == undefined)
    read = !msgHdrForCurrentMessage().isRead;
  MarkSelectedMessagesRead(read);
}

/**
 * Marks the selected messages as flagged or unflagged
 *
 * @param flagged true if trying to mark messages as flagged, false if marking unflagged,
 *        undefined if toggling the flagged status
 */

function MsgMarkAsFlagged(markFlagged)
{
    if (markFlagged == undefined) {
        markFlagged = !SelectedMessagesAreFlagged();
    }
    ReportAnalytics("todo:message:" + (markFlagged ? "flag" : "unflag")); 
    MarkSelectedMessagesFlagged(markFlagged);
}

/**
 * Marks the selected messages as read or unread
 *
 * @param read true if trying to mark messages as read, false if marking unread,
 *        undefined if toggling the read status
 */
function MsgMarkAsPending(pending, aMsgHdr)
{
  if (aMsgHdr == undefined)
    aMsgHdr = msgHdrForCurrentMessage();
  if (pending == undefined)
    pending = !(aMsgHdr.flags & MSG_FLAG_PENDING);
  
  var headers = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
  headers.AppendElement(aMsgHdr);
  
  aMsgHdr.folder.changeMessagesFlag(headers, MSG_FLAG_PENDING, pending);
}

function MsgMarkReadByDate()
{
  window.openDialog( "chrome://messenger/content/markByDate.xul","",
                     "chrome,modal,titlebar,centerscreen",
                     GetLoadedMsgFolder() );
}

function MsgMarkAllRead()
{
  var folder = GetMsgFolderFromUri(GetSelectedFolderURI(), true);
  if (folder)
    folder.markAllMessagesRead();
}

function MsgDownloadFlagged()
{
  gDBView.doCommand(nsMsgViewCommandType.downloadFlaggedForOffline);
}

function MsgDownloadSelected()
{
  gDBView.doCommand(nsMsgViewCommandType.downloadSelectedForOffline);
}

function MsgMarkThreadAsRead()
{
  ClearPendingReadTimer();
  gDBView.doCommand(nsMsgViewCommandType.markThreadRead);
}

function MsgViewPageSource()
{
    var messages = GetSelectedMessages();
    ViewPageSource(messages);
}

function MsgFind()
{
  document.getElementById("FindToolbar").onFindCommand();
}

function MsgFindAgain(reverse)
{
  document.getElementById("FindToolbar").onFindAgainCommand(reverse);
}

function MsgFilters(emailAddress, folder)
{
    if (folder && SmartServerUtils.isSmartFolder(folder))
      folder = null; // ignore smart folders

    if (!folder)
    {
      // try to determine the folder from the selected message. 
      if (gDBView)
      {
   /*
       * Here we face a decision. If the message has been moved to a
       *  different account, then a single filter cannot work for both
       *  manual and incoming scope. So we will create the filter based
       *  on its existing location, which will make it work properly in
       *  manual scope. This is the best solution for POP3 with global
       *  inbox (as then both manual and incoming filters work correctly),
       *  but may not be what IMAP users who filter to a local folder
       *  really want.
       */
        try
        {
          var msgHdr = gDBView.hdrForFirstSelectedMessage;
          folder = msgHdr.folder;
          // except for news, we define the filter on the account's root
          if (!IsNewsMessage(msgHdr))
            folder = folder.rootFolder;
        }
        catch (ex) {}
      }
    }
    var args;
    if (emailAddress)
    {
      /* we have to do prefill filter so we are going to 
         launch the filterEditor dialog
         and prefill that with the emailAddress */
         
      var curFilterList = folder.getFilterList(msgWindow);
      args = {filterList: curFilterList};
      args.filterName = emailAddress;
      window.openDialog("chrome://messenger/content/FilterEditor.xul", "", 
                        "chrome, modal,centerscreen,dialog=yes", args);

      /* if the user hits ok in the filterEditor dialog we set 
         args.refresh=true there
         we check this here in args to show filterList dialog */
      if ("refresh" in args && args.refresh)
      {
         args = { refresh: true, folder: folder };
         MsgFilterList(args);
      }
    }
    else  // just launch filterList dialog
    {
      args = { refresh: false, folder: folder };
      MsgFilterList(args);
    }
}

function MsgApplyFilters()
{
  var filterService = Components.classes["@mozilla.org/messenger/services/filters;1"].getService(Components.interfaces.nsIMsgFilterService);

  var preselectedFolder = GetFirstSelectedMsgFolder();
  var selectedFolders = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
  selectedFolders.AppendElement(preselectedFolder);
         
  var curFilterList = preselectedFolder.getFilterList(msgWindow);
  // create a new filter list and copy over the enabled filters to it.
  // We do this instead of having the filter after the fact code ignore
  // disabled filters because the Filter Dialog filter after the fact
  // code would have to clone filters to allow disabled filters to run,
  // and we don't support cloning filters currently.
  var tempFilterList = filterService.getTempFilterList(preselectedFolder);
  var numFilters = curFilterList.filterCount;
  // make sure the temp filter list uses the same log stream
  tempFilterList.logStream = curFilterList.logStream;
  tempFilterList.loggingEnabled = curFilterList.loggingEnabled;
  var newFilterIndex = 0;
  for (var i = 0; i < numFilters; i++)
  {
    var curFilter = curFilterList.getFilterAt(i);
    // only add enabled, UI visibile filters that are in the manual context
    if (curFilter.enabled && !curFilter.temporary &&
        (curFilter.filterType & Components.interfaces.nsMsgFilterType.Manual))
    {
      tempFilterList.insertFilterAt(newFilterIndex, curFilter);
      newFilterIndex++;
    }
  }
  filterService.applyFiltersToFolders(tempFilterList, selectedFolders, msgWindow);
}

function MsgApplyFiltersToSelection()
{
  var selectedMessages = GetSelectedMessages();
  if (selectedMessages.length) {
    // convert our message uris into an array of msg hdrs
    var msgHdrs = [];
    for each (var uri in selectedMessages)
      msgHdrs.push(messenger.msgHdrFromURI(uri));
    var filterService =
      Components.classes["@mozilla.org/messenger/services/filters;1"]
                .getService(Components.interfaces.nsIMsgFilterService);

    filterService.applyFilters(Components.interfaces.nsMsgFilterType.Manual,
                               toXPCOMArray(msgHdrs,
                                            Components.interfaces.nsIMutableArray),
                               GetFirstSelectedMsgFolder(),
                               msgWindow);
  }
}

function ChangeMailLayout(newLayout)
{
  gPrefBranch.setIntPref("mail.pane_config.dynamic", newLayout);
  MsgToggleRichView(); // toggle into and out of vertical mode
}

function MsgBodyAllowHTML()
{
    gPrefBranch.setBoolPref("mailnews.display.prefer_plaintext", false);
    gPrefBranch.setIntPref("mailnews.display.html_as", 0);
    gPrefBranch.setIntPref("mailnews.display.disallow_mime_handlers", 0);
    MsgReload();
    return true;
}

function MsgBodySanitized()
{
    gPrefBranch.setBoolPref("mailnews.display.prefer_plaintext", false);
    gPrefBranch.setIntPref("mailnews.display.html_as", 3);
    gPrefBranch.setIntPref("mailnews.display.disallow_mime_handlers",
                      gDisallow_classes_no_html);
    MsgReload();
    return true;
}

function MsgBodyAsPlaintext()
{
    gPrefBranch.setBoolPref("mailnews.display.prefer_plaintext", true);
    gPrefBranch.setIntPref("mailnews.display.html_as", 1);
    gPrefBranch.setIntPref("mailnews.display.disallow_mime_handlers", gDisallow_classes_no_html);
    MsgReload();
    return true;
}

function ToggleInlineAttachment(target)
{
    var viewAttachmentInline = !pref.getBoolPref("mail.inline_attachments");
    pref.setBoolPref("mail.inline_attachments", viewAttachmentInline)
    target.setAttribute("checked", viewAttachmentInline ? "true" : "false");
    MsgReload();
}

function MsgReload()
{
    ReloadMessage();
}

function MsgStop()
{
    StopUrls();
}

function MsgSendUnsentMsgs()
{
  // if offline, prompt for sendUnsentMessages
  if (MailOfflineMgr.isOnline())
    SendUnsentMessages();
  else
    MailOfflineMgr.goOnlineToSendMessages(msgWindow);
}

function PrintEnginePrintInternal(messageList, numMessages, doPrintPreview, msgType)
{
    if (numMessages == 0) {
        dump("PrintEnginePrint(): No messages selected.\n");
        return false;
    }

    printEngineWindow = window.openDialog("chrome://messenger/content/msgPrintEngine.xul",
                                          "",
                                          "chrome,dialog=no,all,centerscreen",
                                          numMessages, messageList, statusFeedback, doPrintPreview, msgType, window);
    return true;

}

function PrintEnginePrint()
{
    var messageList = getSelectedConversationMessages() || GetSelectedMessages();
    return PrintEnginePrintInternal(messageList, messageList.length, false, Components.interfaces.nsIMsgPrintEngine.MNAB_PRINT_MSG);
}

function PrintEnginePrintPreview()
{
    var messageList = GetSelectedMessages();
    return PrintEnginePrintInternal(messageList, 1, true, Components.interfaces.nsIMsgPrintEngine.MNAB_PRINTPREVIEW_MSG);
}

function IsMailFolderSelected()
{
    var selectedFolders = GetSelectedMsgFolders();
    var numFolders = selectedFolders.length;
    if(numFolders !=1)
        return false;

    var folder = selectedFolders[0];
    if (!folder)
        return false;

    var server = folder.server;
    var serverType = server.type;

    if((serverType == "nntp"))
        return false;
    else return true;
}

function IsGetNewMessagesEnabled()
{
  // users don't like it when the "Get Msgs" button is disabled
  // so let's never do that. 
  // we'll just handle it as best we can in GetFolderMessages()
  // when they click "Get Msgs" and
  // Local Folders or a news server is selected
  // see bugs #89404 and #111102
  return true;
}

function IsGetNextNMessagesEnabled()
{
    var selectedFolders = GetSelectedMsgFolders();
    var numFolders = selectedFolders.length;
    if(numFolders !=1)
        return false;

    var folder = selectedFolders[0];
    if (!folder)
        return false;

    var server = folder.server;
    var serverType = server.type;

    var menuItem = document.getElementById("menu_getnextnmsg");
    if ((serverType == "nntp") && !folder.isServer) {
        var newsServer = server.QueryInterface(Components.interfaces.nsINntpIncomingServer);
        var menuLabel = gMessengerBundle.getFormattedString("getNextNMessages",
                                                            [ newsServer.maxArticles ]);
        menuItem.setAttribute("label",menuLabel);
        menuItem.removeAttribute("hidden");
        return true;
    }

    menuItem.setAttribute("hidden","true");
    return false;
}

function IsEmptyTrashEnabled()
{
  var folderURI = GetSelectedFolderURI();
  var server = GetServer(folderURI);
  return (server && server.canEmptyTrashOnExit?IsMailFolderSelected():false);
}

function IsCompactFolderEnabled()
{
  var server = GetServer(GetSelectedFolderURI());
  return (server && 
      ((server.type != 'imap') || server.canCompactFoldersOnServer) &&
      isCommandEnabled("cmd_compactFolder"));   // checks e.g. if IMAP is offline
}

var gDeleteButton = null;
var gMarkButton = null;

function SetUpToolbarButtons(uri)
{
    //dump("SetUpToolbarButtons("+uri+")\n");

    // eventually, we might want to set up the toolbar differently for imap,
    // pop, and news.  for now, just tweak it based on if it is news or not.
    var forNews = isNewsURI(uri);

    if(!gDeleteButton) gDeleteButton = document.getElementById("button-delete");

    var buttonToHide = null;
    var buttonToShow = null;

    if (forNews) {
        buttonToHide = gDeleteButton;
    }
    else {
        buttonToShow = gDeleteButton;
    }

    if (buttonToHide) {
        buttonToHide.setAttribute('hidden',true);
    }
    if (buttonToShow) {
        buttonToShow.removeAttribute('hidden');
    }
}

var gMessageBrowser;

function getMessageBrowser()
{
  if (!gMessageBrowser)
    gMessageBrowser = document.getElementById("messagepane");

  return gMessageBrowser;
}

function getMarkupDocumentViewer()
{
  return getMessageBrowser().markupDocumentViewer;
}

function MsgSynchronizeOffline()
{
  window.openDialog("chrome://messenger/content/msgSynchronize.xul",
        "", "centerscreen,chrome,modal,titlebar,resizable=yes",{msgWindow:msgWindow});          
}


function MsgMarkByDate() {}
function MsgOpenAttachment() {}
function MsgUpdateMsgCount() {}
function MsgImport() {}
function MsgSynchronize() {}
function MsgGetSelectedMsg() {}
function MsgGetFlaggedMsg() {}
function MsgSelectThread() {}
function MsgShowFolders(){}
function MsgShowLocationbar() {}
function MsgViewAttachInline() {}
function MsgWrapLongLines() {}
function MsgIncreaseFont() {}
function MsgDecreaseFont() {}
function MsgShowImages() {}
function MsgRefresh() {}
function MsgViewPageInfo() {}
function MsgFirstUnreadMessage() {}
function MsgFirstFlaggedMessage() {}
function MsgAddSenderToAddressBook() {}
function MsgAddAllToAddressBook() {}

function SpaceHit(event)
{
  // forward the space hit on to the open panel
  // see bug #3377
  var panels = document.getElementsByTagName("panel"); 
  for (var i = 0; i < panels.length; i++) 
  {
    if (panels[i].state == "open")
    {
      // if the panel has an onSpaceHit function, call it
      // if no onSpaceHit(), still return and let the panel "eat" the event
      // as we don't want it falling through to the 3 pane
      // see #1515 for details
      if (panels[i].onSpaceHit)
        panels[i].onSpaceHit(event);
      return;
    }
  }
   
  // forward the space hit on to the search result, if displayed deck.
  try 
  {
    if (document.getElementById("displayDeck").selectedPanel == document.getElementById('searchResultsBox'))
    {
      document.getElementById("searchResultsList").spaceHit(event);
      return;
    }
  }
  catch (ex)
  {
  }

  var contentWindow = window.top._content;
  var rssiframe = contentWindow.document.getElementById('_mailrssiframe');

  // if we are displaying an RSS article, we really want to scroll the nested iframe
  if (rssiframe)
    contentWindow = rssiframe.contentWindow;

  if (event && event.shiftKey) {
    // if at the start of the message, go to the previous one
    if (contentWindow.scrollY > 0) {
      contentWindow.scrollByPages(-1);
    }
    else {
      goDoCommand("cmd_previousUnreadMsg");
    }
  }
  else {
    // if at the end of the message, go to the next one
    if (contentWindow.scrollY < contentWindow.scrollMaxY) {
      contentWindow.scrollByPages(1);
    }
    else {
      goDoCommand("cmd_nextUnreadMsg");
    }
  }
}

function IsAccountOfflineEnabled()
{
  var selectedFolders = GetSelectedMsgFolders();

  if (selectedFolders && (selectedFolders.length == 1))
      return selectedFolders[0].supportsOffline;
     
  return false;
}

function GetDefaultAccountRootFolder()
{
  try {
    var account = accountManager.defaultAccount;
    var defaultServer = account.incomingServer;
    var defaultFolder = defaultServer.rootMsgFolder;
    return defaultFolder;
  }
  catch (ex) {
  }
  return null;
}

function GetFolderMessages()
{
  var selectedFolders = GetSelectedMsgFolders();
  var defaultAccountRootFolder = GetDefaultAccountRootFolder();
  
  // if no default account, get msg isn't going do anything anyways
  // so bail out
  if (!defaultAccountRootFolder)
    return;

  // if nothing selected, use the default
  var folder = selectedFolders.length ? selectedFolders[0] : defaultAccountRootFolder;

  var _processFolder = function (folder) {
    var serverType = folder.server.type;

    if (folder.isServer && (serverType == "nntp")) {
      // if we're doing "get msgs" on a news server
      // update unread counts on this server
      folder.server.performExpand(msgWindow);
      return;
    }
    else if (serverType == "none") {
      // if "Local Folders" is selected
      // and the user does "Get Msgs"
      // and LocalFolders is not deferred to,
      // get new mail for the default account
      //
      // XXX TODO
      // should shift click get mail for all (authenticated) accounts?
      // see bug #125885
      if (!folder.server.isDeferredTo)
        folder = defaultAccountRootFolder;
    }

    var folders = new Array(1);
    folders[0] = folder;
    GetNewMessages(folders, folder.server);
  };

  // if we are dealing with a smart folder in a smart account, we need to process
  // every underlying folder searched by that smart folder
  if (SmartServerUtils.isSmartFolder(folder))
  {
    let folders = SmartServerUtils.foldersForSmartFolder(folder);
    for each (var f in folders)
         _processFolder(f);
  }
  else
    _processFolder(folder);
}

function SendUnsentMessages()
{
  var msgSendlater = Components.classes["@mozilla.org/messengercompose/sendlater;1"]
                               .getService(Components.interfaces.nsIMsgSendLater);

  var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
                                 .getService(Components.interfaces.nsIMsgAccountManager);
  var allIdentities = accountManager.allIdentities;
  var identitiesCount = allIdentities.Count();
  for (var i = 0; i < identitiesCount; i++) {
    var currentIdentity = allIdentities.QueryElementAt(i, Components.interfaces.nsIMsgIdentity);
    var msgFolder = msgSendlater.getUnsentMessagesFolder(currentIdentity);
    if (msgFolder) {
      var numMessages = msgFolder.getTotalMessages(false /* include subfolders */);
      if(numMessages > 0) {
        msgSendlater.sendUnsentMessages(currentIdentity);
        // Right now, all identities point to the same unsent messages
        // folder, so to avoid sending multiple copies of the
        // unsent messages, we only call messenger.SendUnsentMessages() once.
        // See bug #89150 for details.
        break;
      }
    }
  }

  // reset the unsent notification monitor
  if ("pbUnsentMessageMonitor" in window)
    pbUnsentMessageMonitor.reset();
}

function CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo)
{
  var outNumFolders = new Object();
  var inboxFolder = currentServer.rootMsgFolder.getFoldersWithFlag(0x1000, 1, outNumFolders); 

  // coalesce the servers that download into the same folder...
  var index = localFoldersToDownloadTo.GetIndexOf(inboxFolder);
  if (index == -1)
  {
    if (inboxFolder) 
    {
      inboxFolder.biffState = Components.interfaces.nsIMsgFolder.nsMsgBiffState_NoMail;
      inboxFolder.clearNewMessages();
    }
    localFoldersToDownloadTo.AppendElement(inboxFolder);
    index = pop3DownloadServersArray.length;
    pop3DownloadServersArray[index] = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
  }
  pop3DownloadServersArray[index].AppendElement(currentServer);
}

function GetMessagesForAllAuthenticatedAccounts()
{
  // now log into any server
  try
  {
    var allServers = accountManager.allServers;
    var i;
    // array of isupportsarrays of servers for a particular folder
    var pop3DownloadServersArray = new Array();
    // parallel isupports array of folders to download to...
    var localFoldersToDownloadTo = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
    var pop3Server;

    for (i = 0; i < allServers.Count(); ++i)
    {
      var currentServer = allServers.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
      var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + currentServer.type].getService(Components.interfaces.nsIMsgProtocolInfo);
      if (protocolinfo.canGetMessages && !currentServer.passwordPromptRequired)
      {
        if (currentServer.type == "pop3")
        {
          CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
          pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
        }
        else
        {
          // get new messages on the server for imap or rss
          GetMessagesForInboxOnServer(currentServer);
        }
      }
    }
    for (i = 0; i < pop3DownloadServersArray.length; ++i)
    {
      // any ol' pop3Server will do - the serversArray specifies which servers to download from
      pop3Server.downloadMailFromServers(pop3DownloadServersArray[i], msgWindow, localFoldersToDownloadTo.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgFolder), null);
    }
  }
  catch(ex)
  {
      dump(ex + "\n");
  }
}

function GetMessagesForAccount(aEvent)
{
  var uri = aEvent.target.id;
  var server = GetServer(uri);
  GetMessagesForInboxOnServer(server);
  aEvent.stopPropagation();
}


function CommandUpdate_UndoRedo()
{
    ShowMenuItem("menu_undo", true);
    EnableMenuItem("menu_undo", SetupUndoRedoCommand("cmd_undo"));
    ShowMenuItem("menu_redo", true);
    EnableMenuItem("menu_redo", SetupUndoRedoCommand("cmd_redo"));
}

function SetupUndoRedoCommand(command)
{
    var loadedFolder = GetLoadedMsgFolder();

    // if we have selected a server, and are viewing account central
    // there is no loaded folder
    if (!loadedFolder)
      return false;

    var server = loadedFolder.server;
    if (!(server.canUndoDeleteOnServer))
      return false;

    var canUndoOrRedo = false;
    var txnType = 0;

    if (command == "cmd_undo")
    {
        canUndoOrRedo = messenger.canUndo();
        txnType = messenger.getUndoTransactionType();
    }
    else
    {
        canUndoOrRedo = messenger.canRedo();
        txnType = messenger.getRedoTransactionType();
    }

    if (canUndoOrRedo)
    {
        switch (txnType)
        {
        default:
        case 0:
            goSetMenuValue(command, 'valueDefault');
            break;
        case 1:
            goSetMenuValue(command, 'valueDeleteMsg');
            break;
        case 2:
            goSetMenuValue(command, 'valueMoveMsg');
            break;
        case 3:
            goSetMenuValue(command, 'valueCopyMsg');
            break;
        }
    }
    else
    {
        goSetMenuValue(command, 'valueDefault');
    }
    return canUndoOrRedo;
}

function HandleJunkStatusChanged(folder)
{
  // this might be the stand alone window, open to a message that was
  // and attachment (or on disk), in which case, we want to ignore it.
  var loadedMessage = GetLoadedMessage();
  if (loadedMessage && (!(/type=application\/x-message-display/.test(loadedMessage))) && IsCurrentLoadedFolder(folder))
  {
    // if multiple message are selected and we change the junk status
    // we don't want to show the junk bar (since the message pane is blank)
    var msgHdr = null;
    if (GetNumSelectedMessages() == 1)
      msgHdr = messenger.msgHdrFromURI(loadedMessage);
    var messageContainer = pbGetMessageContainerForHdr(msgHdr);
    var junkBarWasDisplayed = messageContainer && messageContainer.getElementsByClassName('junkcontentbox').length;
    if (msgHdr && folder)
    {
      var moveJunkMail = (folder && folder.server && folder.server.spamSettings) ?
                          folder.server.spamSettings.manualMark : false;

      var junkScore = msgHdr.getStringProperty("junkscore"); 
      var isJunk = ((junkScore != "") && (junkScore != "0"));
      // If we are about to move the message that was just marked as junk, 
      // then don't bother reloading it and don't bother showing the notification bar.
      if (!(isJunk && moveJunkMail))
      {
        updateJunkStatusForHdr(msgHdr);
        if (messageContainer && junkBarWasDisplayed != messageContainer.getElementsByClassName('junkcontentbox').length)
        {
          // we may be forcing junk mail to be rendered with sanitized html. In that scenario, we want to 
          // reload the message if the status has just changed to not junk. 
          var sanitizeJunkMail = gPrefBranch.getBoolPref("mail.spam.display.sanitize");
          if (sanitizeJunkMail) // only bother doing this if we are modifying the html for junk mail....
          {
            MsgReload();
          }
        }
      }
    }
  }
}

/**
 *  msgHdrForCurrentMessage
 *   Returns the msg hdr associated with the current loaded message.
 */
function msgHdrForCurrentMessage()
{
  var msgURI = GetLoadedMessage();
  return (msgURI && !(/type=application\/x-message-display/.test(msgURI))) ? messenger.msgHdrFromURI(msgURI) : null;
}

/* marks the message as read, but goes through the view */
function MarkCurrentMessageAsRead()
{
  ClearPendingReadTimer();
  gDBView.doCommand(nsMsgViewCommandType.markMessagesRead);
}

/**
 * Mark a specified message as read.
 * @param msgHdr header (nsIMsgDBHdr) of the message to mark as read
 */
function MarkMessageAsRead(msgHdr)
{
  ClearPendingReadTimer();
  var headers = Components.classes["@mozilla.org/supports-array;1"]
  .createInstance(Components.interfaces.nsISupportsArray);
  headers.AppendElement(msgHdr);
  msgHdr.folder.markMessagesRead(headers, true);
}

/**
 * Toggle a specified message as read / unread
 * @param msgHdr header (nsIMsgDBHdr) of the message to mark as read
 */
function ToggleMessageAsRead(msgHdr)
{
  ClearPendingReadTimer();
  var headers = Components.classes["@mozilla.org/supports-array;1"]
  .createInstance(Components.interfaces.nsISupportsArray);
  headers.AppendElement(msgHdr);
  msgHdr.folder.markMessagesRead(headers, !msgHdr.isRead);
}

/**
 * Toggle a specified message as flagged / unflagged
 * @param msgHdr header (nsIMsgDBHdr) of the message to mark as flagged
 */
function ToggleMessageAsFlagged(msgHdr)
{
  var headers = Components.classes["@mozilla.org/supports-array;1"]
  .createInstance(Components.interfaces.nsISupportsArray);
  headers.AppendElement(msgHdr);
  msgHdr.folder.markMessagesFlagged(headers, !msgHdr.isFlagged);
}

/**
 * Toggle a specified message as junk / not junk
 * @param msgHdr header (nsIMsgDBHdr) of the message to mark as junk
 */
function ToggleMessageAsJunk(msgHdr)
{
  var junkScore = msgHdr.getStringProperty("junkscore");
  var isJunk = ((junkScore != "") && (junkScore != "0"));
  gDBView.setMsgHdrJunkStatus(msgHdr, isJunk ? Components.interfaces.nsIJunkMailPlugin.GOOD : Components.interfaces.nsIJunkMailPlugin.JUNK);
}

function ClearPendingReadTimer()
{
  if (gMarkViewedMessageAsReadTimer)
  {
    clearTimeout(gMarkViewedMessageAsReadTimer);
    gMarkViewedMessageAsReadTimer = null;
  }
}

// this is called when layout is actually finished rendering a 
// mail message. OnMsgLoaded is called when libmime is done parsing the message
function OnMsgParsed(aUrl)
{
  goUpdateCommand("cmd_toggleInspectorPane");

  // browser doesn't do this, but I thought it could be a useful thing to test out...
  // If the find bar is visible and we just loaded a new message, re-run 
  // the find command. This means the new message will get highlighted and
  // we'll scroll to the first word in the message that matches the find text.
  var findBar = document.getElementById("FindToolbar");
  if (!findBar.hidden)
    findBar.onFindAgainCommand(false);

  // run the phishing detector on the message
  var msgHdr = msgHdrForCurrentMessage();
  if (msgHdr && !(msgHdr.getUint32Property("notAPhishMessage") || msgHdr.getUint32Property("skipMalware")))
    gPhishingDetector.analyzeMsgForPhishingURLs(aUrl);
  // notify anyone (e.g., extensions) who's interested in when a message is loaded.
  var messageUrl = aUrl.QueryInterface(Components.interfaces.nsIMsgMessageUrl);
  var msgURI = messageUrl.uri ? messageUrl.uri : GetLoadedMessage();
  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.notifyObservers(msgWindow.msgHeaderSink, "MsgMsgDisplayed", msgURI);

  var messagepane = getMessageBrowser();
  var doc = messagepane.contentDocument;
  // XXX TODO
  // is this still necessary?  isn't everything in an iframe now?
  ScaleOverflowingImages(doc);

  var iframes = doc.getElementsByTagName('iframe');
  for (var i = 0; i < iframes.length; i++)
  { 
    if (iframes[i].className == "postbox-conversation-iframe")
    {
      ScaleOverflowingImages(iframes[i].contentDocument);
      
      AdjustConversationHeight(messagepane, iframes[i], iframes[i].contentDocument.body, false /* aOnPaint */, true /* aForceAdjustmentNow */);
    }
  }
}

function ScaleOverflowingImages(doc)
{
  var imgs = doc.getElementsByTagName("img");
  for (var i=0; i < imgs.length; i++)
  {
    img = imgs[i];
    if (img.naturalWidth > doc.body.clientWidth) {
      if (gShrinkToFit) {
        // the user might have shrink to fit enabled, but disabled it for this image (by clicking on it)
        // int that case, don't set isshrunk on resizing the window
        var shrinkToFitEnabledForImage = img.getAttribute("shrinktofit") != "false";
        if (shrinkToFitEnabledForImage) {
          img.setAttribute("isshrunk", "true");
        } else {
          img.setAttribute("overflowing", "true");
        }
      } else {
        img.setAttribute("overflowing", "true");
        img.setAttribute("shrinktofit", "false");
        img.setAttribute("resizecursor", "default");
      }
    }
  }
}
        
function OnMsgLoaded(aUrl, aMsgHdr)
{
    if (!aUrl)
      return;

    // nsIMsgMailNewsUrl.folder throws an error when opening .eml files.
    var folder;
    try {
      folder = aUrl.folder;
    }
    catch (ex) {}

    var msgURI = GetLoadedMessage();

    if (!folder || !msgURI)
      return;

    // If we are in the middle of a delete or move operation, make sure that
    // if the user clicks on another message then that message stays selected
    // and the selection does not "snap back" to the message chosen by
    // SetNextMessageAfterDelete() when the operation completes (bug 243532).
    // But the just loaded message might be getting deleted, if the user
    // deletes it before the message is loaded (bug 183394)
    var wintype = document.documentElement.getAttribute('windowtype');
    if (wintype == "mail:messageWindow" || GetThreadTree().view.selection.currentIndex != gSelectedIndexWhenDeleting)
      gNextMessageViewIndexAfterDelete = -2;

    var msgHdr = msgHdrForCurrentMessage();
    if (msgHdr == aMsgHdr)
      updateJunkStatusForHdr(msgHdr);

    // we just finished loading a message. set a timer to actually mark the message is read after n seconds
    // where n can be configured by the user.
    var markReadAutoMode = gPrefBranch.getBoolPref("mailnews.mark_message_read.auto");
        
    // if we are in the 3 pane, and pb.always_mark_as_read is false
    // don't mark as read if our selection contains a conversation row (a container)
    var markAsRead = true;
    try
    {
      if (wintype == "mail:3pane" && !gPrefBranch.getBoolPref("pb.always_mark_as_read"))
      {
        // can't use GetSelectedIndices(), because the conversation view excludes the dummy row
        var selection = GetThreadTree().view.selection;
        var rangeCount = selection.getRangeCount();
        for (var i = 0; markAsRead && i < rangeCount; i++)
        {
          var startIndex = {};
          var endIndex = {};
          selection.getRangeAt(i, startIndex, endIndex);
          for (var j = startIndex.value; markAsRead && j <= endIndex.value; j++)
          {
            if (gDBView.isContainer(j))
            {
              markAsRead = false;
            }
          }
        }
      }
    }
    catch (ex)
    {
      Components.utils.reportError(ex);
    }
    
    // We just finished loading a message. If messages are to be marked as read
    // automatically, set a timer to mark the message is read after n seconds
    // where n can be configured by the user.
    if (msgHdr && markAsRead && markReadAutoMode)
    {
      let markReadOnADelay = gPrefBranch.getBoolPref("mailnews.mark_message_read.delay");
      
      // Only use the timer if viewing using the 3-pane preview pane and the
      // user has set the pref.
      if (wintype == "mail:3pane") // 3-pane window
      {
        ClearPendingReadTimer();
        let markReadDelayTime = gPrefBranch.getIntPref("mailnews.mark_message_read.delay.interval");
        if (!markReadOnADelay || markReadDelayTime == 0)
        {
          gMarkViewedMessageAsReadTimer = setTimeout(MarkCurrentMessageAsRead, 200);
        }
        else
        {
          gMarkViewedMessageAsReadTimer = setTimeout(MarkCurrentMessageAsRead, markReadDelayTime * 1000);
        }
      }
      else
      {
        MarkCurrentMessageAsRead();
      }
    }

    // See if MDN was requested but has not been sent.
    HandleMDNResponse(aUrl);
}

//
// This function handles all mdn response generation (ie, imap and pop).
// For pop the msg uid can be 0 (ie, 1st msg in a local folder) so no
// need to check uid here. No one seems to set mimeHeaders to null so
// no need to check it either.
//
function HandleMDNResponse(aUrl)
{
  if (!aUrl)
    return;

  var msgFolder = aUrl.folder;
  var msgURI = GetLoadedMessage();
  if (!msgFolder || !msgURI || IsNewsMessage(msgURI))
    return;

  // if the message is marked as junk, do NOT attempt to process a return receipt
  // in order to better protect the user
  if (SelectedMessagesAreJunk())
    return;

  var msgHdr = messenger.msgHdrFromURI(msgURI);
  var mimeHdr;

  try {
    mimeHdr = aUrl.mimeHeaders;
  } catch (ex) {
    return;
  }

  // If we didn't get the message id when we downloaded the message header,
  // we cons up an md5: message id. If we've done that, we'll try to extract
  // the message id out of the mime headers for the whole message.
  var msgId = msgHdr.messageId;
  if (msgId.split(":")[0] == "md5")
  {
    var mimeMsgId = mimeHdr.extractHeader("Message-Id", false);
    if (mimeMsgId)
      msgHdr.messageId = mimeMsgId;
  }

  // After a msg is downloaded it's already marked READ at this point so we must check if
  // the msg has a "Disposition-Notification-To" header and no MDN report has been sent yet.
  var msgFlags = msgHdr.flags;
  if ((msgFlags & MSG_FLAG_IMAP_DELETED) || (msgFlags & MSG_FLAG_MDN_REPORT_SENT))
    return;

  var DNTHeader = mimeHdr.extractHeader("Disposition-Notification-To", false);
  var oldDNTHeader = mimeHdr.extractHeader("Return-Receipt-To", false);
  if (!DNTHeader && !oldDNTHeader)
    return;

  // Everything looks good so far, let's generate the MDN response.
  var mdnGenerator = Components.classes["@mozilla.org/messenger-mdn/generator;1"].
                                  createInstance(Components.interfaces.nsIMsgMdnGenerator);
  mdnGenerator.process(MDN_DISPOSE_TYPE_DISPLAYED, msgWindow, msgFolder, msgHdr.messageKey, mimeHdr, false);

  // Reset mark msg MDN "Sent" and "Not Needed".
  msgHdr.flags = (msgFlags & ~MSG_FLAG_MDN_REPORT_NEEDED);
  msgHdr.OrFlags(MSG_FLAG_MDN_REPORT_SENT);

  // Commit db changes.
  var msgdb = msgFolder.getMsgDatabase(msgWindow);
  if (msgdb)
    msgdb.Commit(ADDR_DB_LARGE_COMMIT);
}

function QuickSearchFocus() 
{
  var quickSearchTextBox = document.getElementById('searchInput');
  if (quickSearchTextBox) {
    quickSearchTextBox.focus();
    quickSearchTextBox.select();
  }
}

function MsgSearchMessages()
{
  var preselectedFolder = null;
  if ("GetFirstSelectedMsgFolder" in window)
    preselectedFolder = GetFirstSelectedMsgFolder();

  var args = { folder: preselectedFolder };
  OpenOrFocusWindow(args, "mailnews:search", "chrome://messenger/content/SearchDialog.xul");
}

function MsgJunkMailInfo(aCheckFirstUse)
{
  if (aCheckFirstUse) {
    if (!pref.getBoolPref("mailnews.ui.junk.firstuse"))
      return;
    pref.setBoolPref("mailnews.ui.junk.firstuse", false);

    // check to see if this is an existing profile where the user has started using
    // the junk mail feature already
    var junkmailPlugin = Components.classes["@mozilla.org/messenger/filter-plugin;1?name=bayesianfilter"]
                                   .getService(Components.interfaces.nsIJunkMailPlugin);
    if (junkmailPlugin.userHasClassified)
      return;
  }

  var desiredWindow = GetWindowByWindowType("mailnews:junkmailinfo");

  if (desiredWindow)
    desiredWindow.focus();
  else
    window.openDialog("chrome://messenger/content/junkMailInfo.xul", "mailnews:junkmailinfo", "centerscreen,resizeable=no,titlebar,chrome,modal", null);
}

function MsgSearchAddresses()
{
  var args = { directory: null };
  OpenOrFocusWindow(args, "mailnews:absearch", "chrome://messenger/content/ABSearchDialog.xul");
}
 
function MsgFilterList(args)
{
  OpenOrFocusWindow(args, "mailnews:filterlist", "chrome://messenger/content/FilterListDialog.xul");
}

function GetWindowByWindowType(windowType)
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
  return windowManager.getMostRecentWindow(windowType);
}

function OpenOrFocusWindow(args, windowType, chromeURL)
{
  var desiredWindow = GetWindowByWindowType(windowType);

  if (desiredWindow) {
    desiredWindow.focus();
    if ("refresh" in args && args.refresh)
      desiredWindow.refresh();
  }
  else
    window.openDialog(chromeURL, "", "chrome,resizable,status,centerscreen,dialog=no", args);
}

function buildHelpMenu()
{
  var enableLicenseUI = pref.getBoolPref("pb.enable_license_ui");
  var menuLicenseSep = document.getElementById("menu_license_sep");
  if (menuLicenseSep)
    menuLicenseSep.setAttribute("hidden", enableLicenseUI ? "" : "true");
  var menuLicense = document.getElementById("menu_license");
    menuLicense.setAttribute("hidden", enableLicenseUI ? "" : "true");
}

#include pbMailWindowOverlay.js
