# -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Communicator client code, released
# March 31, 1998.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998-1999
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Jan Varga (varga@nixcorp.com)
#   HÂkan Waara (hwaara@chello.se)
#   Neil Rashbrook (neil@parkwaycc.co.uk)
#   Seth Spitzer <sspitzer@netscape.com>
#   David Bienvenu <bienvenu@nventure.com>
#   Jeremy Morton <bugzilla@game-point.net>
#   Postbox, Inc. <contact@postbox-inc.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

Components.utils.import("resource:///modules/activity/activityModules.js");
Components.utils.import("resource:///modules/mailnewsMigrator.js");
Components.utils.import("resource:///modules/pbFavoriteContacts.js");
Components.utils.import("resource://gre/modules/PluralForm.jsm");

/* This is where functions related to the 3 pane window are kept */

// from MailNewsTypes.h
const nsMsgKey_None = 0xFFFFFFFF;
const nsMsgViewIndex_None = 0xFFFFFFFF;
const kMailCheckOncePrefName = "mail.startup.enabledMailCheckOnce";

const kStandardPaneConfig = 0;
const kWidePaneConfig = 1;
const kVerticalPaneConfig = 2;

const kNumFolderViews = 4; // total number of folder views

// from nsMsgFolderFlags.h
const MSG_FOLDER_FLAG_ELIDED = 0x10;

var gFolderTree;
var gAccountTree;
var gFolderView;
var gAccountView;
var gMessagePane;
var gThreadTree;
var gThreadAndMessagePaneSplitter = null;
var gUnreadCount = null;
var gTotalCount = null;
var gCurrentLoadingFolderURI;
var gCurrentFolderToReroot;
var gCurrentLoadingFolderSortType = 0;
var gCurrentLoadingFolderSortOrder = 0;
var gCurrentLoadingFolderViewType = 0;
var gCurrentLoadingFolderViewFlags = 0;
var gRerootOnFolderLoad = false;
var gNextMessageAfterDelete = null;
var gNextMessageAfterLoad = null;
var gNextMessageViewIndexAfterDelete = -2;
var gSelectedIndexWhenDeleting = -1;
var gCurrentlyDisplayedMessage=nsMsgViewIndex_None;
var gStartFolderUri = null;
var gStartMsgKey = nsMsgKey_None;
var gRightMouseButtonDown = false;
// Global var to keep track of which row in the thread pane has been selected
// This is used to make sure that the row with the currentIndex has the selection
// after a Delete or Move of a message that has a row index less than currentIndex.
var gThreadPaneCurrentSelectedIndex = -1;
var gLoadStartFolder = true;
var gNewAccountToLoad = null;

// Global var to keep track of if the 'Delete Message' or 'Move To' thread pane
// context menu item was triggered.  This helps prevent the tree view from
// not updating on one of those menu item commands.
var gThreadPaneDeleteOrMoveOccurred = false;

//If we've loaded a message, set to true.  Helps us keep the start page around.
var gHaveLoadedMessage;

var gDisplayStartupPage = false;

var gNotifyDefaultInboxLoadedOnStartup = false;

var gNoWizard = true;

function SelectAndScrollToKey(aMsgKey)
{
  // select the desired message
  // if the key isn't found, we won't select anything
  gDBView.selectMsgByKey(aMsgKey);

  // is there a selection?
  // if not, bail out.
  var indicies = GetSelectedIndices(gDBView);
  if (!indicies || !indicies.length)
    return false;

  // now scroll to it
  EnsureRowInThreadTreeIsVisible(indicies[0]);
  return true;
}

// A helper routine called after a folder is loaded to make sure
// we select and scroll to the correct message (could be the first new message,
// could be the last displayed message, etc.)
// returns true if we ended up scrolling to a message
function ScrollToMessageAfterFolderLoad (folder)
{
  var scrolled = pref.getBoolPref("mailnews.scroll_to_new_message") &&
      ScrollToMessage(nsMsgNavigationType.firstNew, true, false /* selectMessage */);
  if (!scrolled && folder && pref.getBoolPref("mailnews.remember_selected_message"))
  {
    // if we failed to scroll to a new message,
    // reselect the last selected message
    var lastMessageLoaded = folder.lastMessageLoaded;
    if (lastMessageLoaded != nsMsgKey_None)
      scrolled = SelectAndScrollToKey(lastMessageLoaded);
  }

  if (!scrolled && gDBView)
  {
    // if we still haven't scrolled,
    // scroll to the newest, which might be the top or the bottom
    // depending on our sort order and sort type
    if (gDBView.sortOrder == nsMsgViewSortOrder.ascending)
    {
      switch (gDBView.sortType)
      {
        case nsMsgViewSortType.byDate:
        case nsMsgViewSortType.byReceived:
        case nsMsgViewSortType.byId:
        case nsMsgViewSortType.byThread:
         scrolled = ScrollToMessage(nsMsgNavigationType.lastMessage, true, false /* selectMessage */);
         break;
      }
    }

    // if still we haven't scrolled,
    // scroll to the top.
    if (!scrolled)
      EnsureRowInThreadTreeIsVisible(0);
  }

  return scrolled;
}

// the folderListener object
var folderListener = {
    OnItemAdded: function(parentItem, item) {
      FolderListenerOnItemAdded(item);
    },
  
    OnItemRemoved: function(parentItem, item) { },

    OnItemPropertyChanged: function(item, property, oldValue, newValue) { },

    OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {
      if (item == gMsgFolderSelected) {
        if(property.toString() == "TotalMessages" || property.toString() == "TotalUnreadMessages")
          UpdateStatusMessageCounts(gMsgFolderSelected);
      }
    },

    OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) { },

    OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue) { },
    
    OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) { },

    OnItemEvent: function(folder, event) {
       var eventType = event.toString();
       if (eventType == "FolderLoaded") {
         if (folder) {
           viewDebug("FolderLoaded event with " + folder.URI + "\n");
           var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
           var uri = folder.URI;
           var rerootingFolder = (uri == gCurrentFolderToReroot);
           if (rerootingFolder) {
             gCurrentFolderToReroot = null;
             if(msgFolder) {
               msgFolder.endFolderLoading();
               UpdateStatusQuota(msgFolder);
               // suppress command updating when rerooting the folder
               // when rerooting, we'll be clearing the selection
               // which will cause us to update commands.
               if (gDBView) {
                 gDBView.suppressCommandUpdating = true;
                 // if the db's view isn't set, something went wrong and we should reroot
                 // the folder, which will re-open the view.
                 if (!gDBView.db)
                   gRerootOnFolderLoad = true;
               }
               if (gRerootOnFolderLoad)
                 RerootFolder(msgFolder, gCurrentLoadingFolderViewType, gCurrentLoadingFolderViewFlags, gCurrentLoadingFolderSortType, gCurrentLoadingFolderSortOrder);

               var db = msgFolder.getMsgDatabase(msgWindow);
               if (db)
                 db.resetHdrCacheSize(100);

               if (gDBView) {
                 gDBView.suppressCommandUpdating = false;
               }

               gCurrentLoadingFolderSortType = 0;
               gCurrentLoadingFolderSortOrder = 0;
               gCurrentLoadingFolderViewType = 0;
               gCurrentLoadingFolderViewFlags = 0;

               var scrolled = false;

               LoadCurrentlyDisplayedMessage();  //used for rename folder msg loading after folder is loaded.

               if (gStartMsgKey != nsMsgKey_None) {
                 scrolled = SelectAndScrollToKey(gStartMsgKey);
                 gStartMsgKey = nsMsgKey_None;
               }

               if (gNextMessageAfterLoad) {
                 var type = gNextMessageAfterLoad;
                 gNextMessageAfterLoad = null;

                 // scroll to and select the proper message
                 scrolled = ScrollToMessage(type, true, true /* selectMessage */);
               }
             }
           }
           if (uri == gCurrentLoadingFolderURI) {
             viewDebug("uri == current loading folder uri\n");
             gCurrentLoadingFolderURI = "";
             // scroll to message for virtual folders is done in 
             // gSearchNotificationListener.OnSearchDone (see searchBar.js)
             if (!scrolled && !(gMsgFolderSelected && gMsgFolderSelected.flags & MSG_FOLDER_FLAG_VIRTUAL))
               scrolled = ScrollToMessageAfterFolderLoad(msgFolder);   
             SetBusyCursor(window, false);
           }
           if (gNotifyDefaultInboxLoadedOnStartup && (folder.flags & 0x1000))
           {
              var defaultAccount = accountManager.defaultAccount;
              var defaultServer = defaultAccount.incomingServer;
              var inboxFolder = GetInboxFolder(defaultServer);
              if (inboxFolder && inboxFolder.URI == folder.URI)
              {
                NotifyObservers(null,"defaultInboxLoadedOnStartup",null);
                gNotifyDefaultInboxLoadedOnStartup = false;
              }
           }

           //folder loading is over, now issue quick search if there is an email address
           viewDebug("in folder loaded gQuickSearchController.virtualFolderTerms = " + gQuickSearchController.virtualFolderTerms + "\n");
           if (gMsgFolderSelected)
             viewDebug("in folder loaded gMsgFolderSelected = " + gMsgFolderSelected.URI + "\n");
           if (rerootingFolder)
           {
             if (gQuickSearchController.virtualFolderTerms)
             {
                viewDebug("selected folder is a saved search\n");
                gDBView.viewFolder = gMsgFolderSelected;
                gFocusPane.searchScope = gDBView.viewFolder.URI;
                viewDebug("doing a search() on a saved search = " + gDBView.viewFolder.URI + "\n");
                gQuickSearchController.search();
             }
             else if (gMsgFolderSelected && gMsgFolderSelected.flags & MSG_FOLDER_FLAG_VIRTUAL)
             {
                viewDebug("selected folder is a xfvf\n");
                if (gFocusPane.hasFocusText())
                {
                  viewDebug("doing a search() on a xfvf = " + gDBView.viewFolder.URI + "\n");
                  gQuickSearchController.search();
                }
             }
             else
             {
               // get the view value from the folder
               if (msgFolder)
               {
                 // if our new view is the same as the old view and we already have the list of search terms built up
                 // for the old view, just re-use it. Skip this step if we are in a global search as we never need to re-run the search
                 // in this case. See Postbox Bug #2943.
                 viewDebug("focusTerms = " + gQuickSearchController.focusTerms + " scope = " + gFocusPane.searchScope + "\n");
                 if (gQuickSearchController.focusTerms && gFocusPane.searchScope != kAllMailFoldersURI)
                 {
                    viewDebug("searching gDefaultSearchViewTerms and rerootingFolder\n");
                    // fake the notifications to make us reload out default filter terms.
                    // mscott - I don't think this is quite right...
                    //gSaveDefaultSVTerms = gDefaultSearchViewTerms;
                    gQuickSearchController.search();
                 }
               }
             }
           }
         }
       }
       else if (eventType == "ImapHdrDownloaded") {
         if (folder) {
           var imapFolder = folder.QueryInterface(Components.interfaces.nsIMsgImapMailFolder);
           if (imapFolder) {
            var hdrParser = imapFolder.hdrParser;
            if (hdrParser) {
              var msgHdr = hdrParser.GetNewMsgHdr();
              if (msgHdr)
              {
                var hdrs = hdrParser.headers;
                if (hdrs && hdrs.indexOf("X-attachment-size:") > 0) {
                  msgHdr.OrFlags(0x10000000);  // 0x10000000 is MSG_FLAG_ATTACHMENT
                }
                if (hdrs && hdrs.indexOf("X-image-size:") > 0) {
                  msgHdr.setStringProperty("imageSize", "1");
                }
              }
            }
           }
         }
       }
       else if (eventType == "DeleteOrMoveMsgCompleted") {
         HandleDeleteOrMoveMsgCompleted(folder);
       }
       else if (eventType == "DeleteOrMoveMsgFailed") {
         HandleDeleteOrMoveMsgFailed(folder);
       }
       else if (eventType == "AboutToCompact") {
        if (gDBView)
          gCurrentlyDisplayedMessage = gDBView.currentlyDisplayedMessage;
       }
       else if (eventType == "CompactCompleted") {
         HandleCompactCompleted(folder);
       }
       else if (eventType == "RenameCompleted") {
         SelectFolder(folder.URI);
       }
       else if (eventType == "JunkStatusChanged") {
         HandleJunkStatusChanged(folder);
       }
    }
}

var folderViewObserver = {
    canDrop: function(index, orientation)
    {
      return CanDropOnFolderTree(index, orientation);
    },

    onDrop: function(row, orientation)
    {
      DropOnFolderTree(row, orientation);
    },

    onToggleOpenState: function(index)
    {
      var folderTree = GetFolderTree();

      // Nothing to do when collapsing an item.
      if (folderTree.view.isContainerOpen(index))
        return;

      var folder = GetFolderResource(folderTree, index);

      if (folderTree.view.getLevel(index) == 0)
      {
        // (Imap/Nntp/Pop) Account item.
        folder.server.performExpand(msgWindow);
      }
      else if (folder instanceof Components.interfaces.nsIMsgImapMailFolder)
      {
        // Imap message folder item.
        folder.performExpand(msgWindow);
      }
    },

    onCycleHeader: function(colID, elt)
    {
    },

    onCycleCell: function(row, colID)
    {
    },

    onSelectionChanged: function()
    {
      document.getElementById('focusPaneByFolderHeader').setAttribute('firstrowselected', GetSelectedFolderIndex() == 0);
      // every time selection in the folder pane changes, notify the favorites bar
      setTimeout(function() {gFavoritesBar.onSelectionChanged();}, 0);
    },

    onPerformAction: function(action)
    {
    },

    onPerformActionOnRow: function(action, row)
    {
    },

    onPerformActionOnCell: function(action, row, col)
    {
    },

  onRowCountChange: function (aNewRowCount) 
  {
  },
}

function HandleDeleteOrMoveMsgFailed(folder)
{
  gDBView.onDeleteCompleted(false);
  if(IsCurrentLoadedFolder(folder)) {
    if(gNextMessageAfterDelete) {
      gNextMessageAfterDelete = null;
      gNextMessageViewIndexAfterDelete = -2;
    }
  }

  // fix me???
  // ThreadPaneSelectionChange(true);
}

// WARNING
// this is a fragile and complicated function.
// be careful when hacking on it.
// don't forget about things like different imap
// delete models, multiple views (from multiple thread panes,
// search windows, stand alone message windows)
function HandleDeleteOrMoveMsgCompleted(folder)
{
  // you might not have a db view.  this can happen if
  // biff fires when the 3 pane is set to account central.
  if (!gDBView)
    return;

  // if the current tab is not a folder tab, bail out of this
  var tabmail = document.getElementById('tabmail');
  if (tabmail && tabmail.currentTabOwner && tabmail.currentTabOwner.type != 'folder')
    return;

  gDBView.onDeleteCompleted(true);

  if (!IsCurrentLoadedFolder(folder)) {
    // default value after delete/move/copy is over
    gNextMessageViewIndexAfterDelete = -2;
    return;
  }

  var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
  var treeSelection = treeView.selection;

  var indicies = GetSelectedIndices(gDBView);
  
  if (gNextMessageViewIndexAfterDelete == -2) {
    // a move or delete can cause our selection can change underneath us.
    // this can happen when the user
    // deletes message from the stand alone msg window
    // or the search view, or another 3 pane
    if (indicies.length == 0) {
      // this can happen if you double clicked a message
      // in the thread pane, and deleted it from the stand alone msg window
      // see bug #172392
      treeSelection.clearSelection();
      setTitleFromFolder(folder, null);
      ClearMessagePane();
      UpdateMailToolbar("delete from another view, 0 rows now selected");
    }
    else if (indicies.length == 1) {
      // this can happen if you had two messages selected
      // in the thread pane, and you deleted one of them from another view
      // (like the view in the stand alone msg window)
      // since one item is selected, we should load it.

      // if we are in restricted mode, don't update the selection
      // as that will throw away our quick reply
      // see bug #6980 for details
      var inRestrictedMode = false;
      try
      {
        inRestrictedMode = gPbActionMonitor.inRestrictedTabMode(document.getElementById('tabmail').currentTabOwner);
      }
      catch (ex)
      {
      }
      
      if (!inRestrictedMode)
      {
        // select the selected item, so we'll load it
        treeSelection.select(indicies[0]);
        treeView.selectionChanged();
      
        EnsureRowInThreadTreeIsVisible(indicies[0]);
      }
      
      UpdateMailToolbar("delete from another view, 1 row now selected");
    }
    else {
      // this can happen if you have more than 2 messages selected
      // in the thread pane, and you deleted one of them from another view
      // (like the view in the stand alone msg window)
      // since multiple messages are still selected, do nothing.
    }
  }
  else {
    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None)
    {
      var viewSize = treeView.rowCount;
      if (gNextMessageViewIndexAfterDelete >= viewSize)
      {
        if (viewSize > 0)
          gNextMessageViewIndexAfterDelete = viewSize - 1;
        else
        {
          gNextMessageViewIndexAfterDelete = nsMsgViewIndex_None;

          // there is nothing to select since viewSize is 0
          treeSelection.clearSelection();
          setTitleFromFolder(folder, null);
          ClearMessagePane();
          UpdateMailToolbar("delete from current view, 0 rows left");
        }
      }
    }

    // if we are about to set the selection with a new element then DON'T clear
    // the selection then add the next message to select. This just generates
    // an extra round of command updating notifications that we are trying to
    // optimize away.
    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None)
    {
      // when deleting a message we don't update the commands
      // when the selection goes to 0
      // (we have a hack in nsMsgDBView which prevents that update)
      // so there is no need to
      // update commands when we select the next message after the delete;
      // the commands already
      // have the right update state...
      gDBView.suppressCommandUpdating = true;

      // This check makes sure that the tree does not perform a
      // selection on a non selected row (row < 0), else assertions will
      // be thrown.
      if (gNextMessageViewIndexAfterDelete >= 0)
        treeSelection.select(gNextMessageViewIndexAfterDelete);

      // if gNextMessageViewIndexAfterDelete has the same value
      // as the last index we had selected, the tree won't generate a
      // selectionChanged notification for the tree view. So force a manual
      // selection changed call.
      // (don't worry it's cheap if we end up calling it twice).
      if (treeView)
        treeView.selectionChanged();

      EnsureRowInThreadTreeIsVisible(gNextMessageViewIndexAfterDelete);
      gDBView.suppressCommandUpdating = false;

      // hook for extra toolbar items
      // XXX TODO
      // I think there is a bug in the suppression code above.
      // what if I have two rows selected, and I hit delete,
      // and so we load the next row.
      // what if I have commands that only enable where
      // exactly one row is selected?
      UpdateMailToolbar("delete from current view, at least one row selected");
    }
  }

  // default value after delete/move/copy is over
  gNextMessageViewIndexAfterDelete = -2;
}

function HandleCompactCompleted(folder)
{
  if (folder)
  {
    var resource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
    if (resource)
    {
      var uri = resource.Value;
      var msgFolder = msgWindow.openFolder;
      if (msgFolder && uri == msgFolder.URI)
      {
        var msgdb = msgFolder.getMsgDatabase(msgWindow);
        if (msgdb)
        {
          var dbFolderInfo = msgdb.dBFolderInfo;
          sortType = dbFolderInfo.sortType;
          sortOrder = dbFolderInfo.sortOrder;
          viewFlags = dbFolderInfo.viewFlags;
          viewType = dbFolderInfo.viewType;
          dbFolderInfo = null;
        }

        RerootFolder(msgFolder, viewType, viewFlags, sortType, sortOrder);
        LoadCurrentlyDisplayedMessage();
      }
    }
  }
}

function LoadCurrentlyDisplayedMessage()
{
  if (gCurrentlyDisplayedMessage != nsMsgViewIndex_None)
  {
    var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
    var treeSelection = treeView.selection;
    treeSelection.select(gCurrentlyDisplayedMessage);
    if (treeView)
      treeView.selectionChanged();
    EnsureRowInThreadTreeIsVisible(gCurrentlyDisplayedMessage);
    SetFocusThreadPaneIfNotOnMessagePane();
    gCurrentlyDisplayedMessage = nsMsgViewIndex_None; //reset
  }
}

function IsCurrentLoadedFolder(folder)
{
  var msgfolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
  if(msgfolder)
  {
    var folderResource = msgfolder.QueryInterface(Components.interfaces.nsIRDFResource);
    if(folderResource)
    {
      var folderURI = folderResource.Value;
      var currentLoadedFolder = GetThreadPaneFolder();
      if (currentLoadedFolder)
      {
        if (currentLoadedFolder.flags & MSG_FOLDER_FLAG_VIRTUAL)
        {
          var msgDatabase = currentLoadedFolder.getMsgDatabase(msgWindow);
          var dbFolderInfo = msgDatabase.dBFolderInfo;
          return dbFolderInfo.getCharPtrProperty("searchFolderUri").split("|").indexOf(folderURI) != -1;
        }
        var currentURI = currentLoadedFolder.URI;
        return(currentURI == folderURI);
      }
    }
  }

  return false;
}

function ServerContainsFolder(server, folder)
{
  if (!folder || !server)
    return false;

  return server.equals(folder.server);
}

function SelectServer(server)
{
  SelectFolder(server.rootFolder.URI);
}

// we have this incoming server listener in case we need to
// alter the folder pane selection when a server is removed
// or changed (currently, when the real username or real hostname change)
var gThreePaneIncomingServerListener = {
    onServerLoaded: function(server) {},
    onServerUnloaded: function(server) {
      // always reset the folder pane when a server is deleted
      if (!SmartServerUtils.isSmartServer(aServer))
        return ResetFolderPaneSelectionAfterServerUnload();
    },
    onServerChanged: function(server) {
    }
}

// aMsgWindowInitialized: false if we are calling from the onload handler, otherwise true
function UpdateMailPaneConfig(aMsgWindowInitialized) {
  const dynamicIds = ["messagesBox", "mailContent", "threadPaneBox"];
  const layouts = ["standard", "wide", "vertical"];
  var layoutView = gPrefBranch.getIntPref("mail.pane_config.dynamic");
  // we don't support the wide view
  if (layoutView == kWidePaneConfig)
    layoutView = kStandardPaneConfig;
  var desiredId = dynamicIds[layoutView];
  document.getElementById("mailContent")
          .setAttribute("layout", layouts[layoutView]);
  var messagePane = GetMessagePane();
  if (messagePane.parentNode.id != desiredId) {
    ClearAttachmentList();
    var messagePaneSplitter = GetThreadAndMessagePaneSplitter();
    var desiredParent = document.getElementById(desiredId);
    // See Bug 381992. The ctor for the browser element will fire again when we
    // re-insert the messagePaneBox back into the document.
    // But the dtor doesn't fire when the element is removed from the document.
    // Manually call destroy here to avoid a nasty leak.
    document.getElementById("messagepane").destroy();
    desiredParent.appendChild(messagePaneSplitter);
    desiredParent.appendChild(messagePane);
    messagePaneSplitter.orient = desiredParent.orient;
    if (aMsgWindowInitialized)
    {
      messenger.setWindow(null, null);
      messenger.setWindow(window, msgWindow);
      // force the status feedback object to register as a listener on the new msg pane docshell again
      msgWindow.domWindow = window;
      msgWindow.statusFeedback = statusFeedback;
      // our status bar elements have just been re-rooted so fetch them again
      window.MsgStatusFeedback._statusText = document.getElementById("statusbar-display");
      window.MsgStatusFeedback._progressBar = document.getElementById("statusbar-icon");
      if (gDBView && GetNumSelectedMessages() == 1)
        gDBView.reloadMessage();
    }
  }

  document.getElementById('threadTree').pbRichTree = layoutView == kVerticalPaneConfig;
  document.getElementById('pbVerticalSortByHeader').pbRichTree = layoutView == kVerticalPaneConfig;
  document.getElementById('threadTreeStack').setAttribute('vertical', layoutView == kVerticalPaneConfig);
}

const MailPrefObserver = {
  observe: function(subject, topic, prefName) {
    // verify that we're changing the mail pane config pref
    if (topic == "nsPref:changed")
    {
      if (prefName == "mail.pane_config.dynamic")
        UpdateMailPaneConfig(true);
      else if (prefName == "pb.mail_chrome.fontsize" || prefName == "pb.thread_pane.color_tag_rows" || prefName == "pb.thread_pane.vertical.bold_unread")
        UpdateFontAndColorStyles();
#ifdef XP_MACOSX
      else if (prefName == "pb.toolbar.customization.mode" || prefName == "pb.toolbar.customization.iconsize")
      {
        let val = pref.getCharPref('pb.toolbar.customization.mode');
        pbUpdateToolbarMode('mail-bar7', val, pref.getCharPref('pb.toolbar.customization.iconsize'));
        document.getElementById('messengerWindow').setAttribute("color", val == 'monochrome-full' || val == 'monochrome-icons' ? 'mono' : '');
        GetFolderTree().treeBoxObject.clearStyleAndImageCaches();
        GetFolderTree().treeBoxObject.invalidate();
        GetAccountTree().treeBoxObject.clearStyleAndImageCaches();
        GetAccountTree().treeBoxObject.invalidate();
        GetThreadTree().treeBoxObject.clearStyleAndImageCaches();
        GetThreadTree().treeBoxObject.invalidate();
      }
#endif
    }
  }
};

function AutoConfigWizard(okCallback)
{
  NewMailAccount(msgWindow, okCallback);
}

function OnLoadMessenger()
{
  migrateMailnews();

  installGmailKeyBindings();

  GetThreadTree().treeBoxObject.twistyImageGenerator = gPbTwistyGenerator;

  // update the pane config before we exit onload otherwise the user may see a flicker if we poke the document
  // in delayedOnLoadMessenger...
  UpdateMailPaneConfig(false);
  setupEvernoteButton();
  gFocusPane.initFor3Pane();  
  document.loadBindingDocument('chrome://global/content/bindings/textbox.xml');

  // Set a sane starting width/height for all resolutions on new profiles. Do this before the window loads
  if (!document.documentElement.hasAttribute("width"))
  {
    var defaultWidth, defaultHeight;
    if (screen.availHeight <= 600)
    {
      document.documentElement.setAttribute("sizemode", "maximized");
      defaultWidth = 800;
      defaultHeight = 565;
    }
    else // for higher resolution displays, use larger values for height and width
    {
      defaultWidth = screen.availWidth <= 1024 ? screen.availWidth * .95 : screen.availWidth * .8;
      defaultHeight = screen.availHeight * .8;
    }

    document.documentElement.setAttribute("width", defaultWidth);
    document.documentElement.setAttribute("height", defaultHeight);
  }

  UpdateSearchBarSplitterState();
  
  window.addEventListener("MozEnterNativeFullScreen", onEnterFullScreenMode, false);
  window.addEventListener("MozExitNativeFullScreen", onExitFullScreenMode, false);

  gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  gPrefBranch.addObserver("mail.pane_config.dynamic", MailPrefObserver, false);
  gPrefBranch.addObserver("pb.mail_chrome.fontsize", MailPrefObserver, false);
  gPrefBranch.addObserver("pb.thread_pane.color_tag_rows", MailPrefObserver, false);
  gPrefBranch.addObserver("pb.thread_pane.vertical.bold_unread", MailPrefObserver, false);
#ifdef XP_MACOSX
  gPrefBranch.addObserver("pb.toolbar.customization.mode", MailPrefObserver, false);
  gPrefBranch.addObserver("pb.toolbar.customization.iconsize", MailPrefObserver, false);
#endif

  UpdateFontAndColorStyles();

  MailOfflineMgr.init();
  CreateMailWindowGlobals();
  GetMessagePane().collapsed = true;

  // verifyAccounts returns true if the callback won't be called
  // We also don't want the account wizard to open if any sort of account exists
  gNoWizard = verifyAccounts(LoadPostAccountWizard, false, AutoConfigWizard);
  if (gNoWizard)
    LoadPostAccountWizard();
  else
  {
    var favoritesBar = document.getElementById("favoritesBar"); 
    if (favoritesBar)
      favoritesBar.setAttribute("noaccounts", true);
    document.getElementById('tabmail-container').setAttribute("noaccounts", true);
    document.commandDispatcher.updateCommands('mail-toolbar');
  }
}

// sdbm hash from
// http://www.cse.yorku.ca/~oz/hash.html
function sdbm_hash(str)
{
  var hash = 0;
  for (var i = 0; i < str.length; i++) {
    hash = str.charCodeAt(i) + (hash << 6) + (hash << 16) - hash;
  }
  return hash;
}

// HACK
// stomp on the analytic value
function SetAnalytic(aName, aValue)
{
  var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch(null);
  if (prefs.getBoolPref("pb.analytics_service.enabled") == false)
    return;
  
  prefs.setIntPref("pb.analytics." + sdbm_hash(aName), aValue);
}

function RecordUIAnalytics()
{
  SetAnalytic("focuspane:collapsed", IsFocusPaneCollapsed() ? 1 : 0);
  SetAnalytic("inspectorpane:collapsed", IsInspectorPaneCollapsed() ? 1 : 0);
  SetAnalytic("messagepane:collapsed", IsMessagePaneCollapsed() ? 1 : 0);
  SetAnalytic("folderpane:collapsed", IsFolderPaneCollapsed() ? 1 : 0);
}
        
/* Functions related to startup */
function LoadPostAccountWizard()
{
  var favoritesBar = document.getElementById("favoritesBar"); 
  if (favoritesBar)
    favoritesBar.removeAttribute("noaccounts");
  document.getElementById('tabmail-container').removeAttribute("noaccounts");
  gNoWizard = true;

  if (!haveAtLeastOneAccount())
  {
    setTimeout(goQuitApplication, 0);
    return;
  }
    
  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.addObserver(gPb3PaneWindowObserver, "pb-validation-change", false);
  observerService.addObserver(gPb3PaneWindowObserver, "pb-languages-loaded", false);
  observerService.addObserver(gPb3PaneWindowObserver, "pb-label-changed", false);
  observerService.addObserver(gPb3PaneWindowObserver, "quit-application-requested", false);

  InitMsgWindow();
  messenger.setWindow(window, msgWindow);

  InitPanes();
  MigrateAttachmentDownloadStore();
  MigrateJunkMailSettings();
  MigrateArchiveSettings(); // postbox used to use an older archive pref for imap
  // migrate signatures if necessary
  if (gPrefBranch.getIntPref('pb.signatures.version') < Components.interfaces.pbISignatureService.kCurrentVersion)
    Components.classes["@postbox-inc.com/signatures;1"].getService(Components.interfaces.pbISignatureService).migrate();

  Components.utils.import("resource:///modules/mailMigrator.js");
  MailMigrator.migrateMail();
  accountManager.setSpecialFolders();
  accountManager.loadVirtualFolders();
  accountManager.addIncomingServerListener(gThreePaneIncomingServerListener);

  gPhishingDetector.init();
  WebRunner.startup(document.getElementById('messagepane'));
  AddToSession();

  //need to add to session before trying to load start folder otherwise listeners aren't
  //set up correctly.
  // argument[0] --> folder uri
  // argument[1] --> optional message key
  // argument[2] --> optional email address; //will come from aim; needs to show msgs from buddy's email address
  if ("arguments" in window)
  {
    // filter our any feed urls that came in as arguments to the new window...
    if (window.arguments.length && /^feed:/i.test(window.arguments[0] ))
    {
      var feedHandler = Components.classes["@mozilla.org/newsblog-feed-downloader;1"].getService(Components.interfaces.nsINewsBlogFeedDownloader);
      if (feedHandler)
        feedHandler.subscribeToFeed(window.arguments[0], null, msgWindow);
      gStartFolderUri = null;
    }
    else
      gStartFolderUri = (window.arguments.length > 0) ? window.arguments[0] : null;
    gStartMsgKey = (window.arguments.length > 1) ? window.arguments[1]: nsMsgKey_None;
  }

  function completeStartup() {
    CheckValidation(window, false /* aForce */);
    var nsIShellService = Components.interfaces.nsIShellService;
    var shellService;
    var defaultAccount;
    try {
      shellService = Components.classes["@mozilla.org/mail/shell-service;1"].getService(nsIShellService);
      defaultAccount = accountManager.defaultAccount;
    } catch (ex) {}

    // show the default client dialog only if we have at least one account,
    // if we should check for the default client,
    // and we aren't already the default for mail)
    if (shellService && defaultAccount && shellService.shouldCheckDefaultClient
        && !shellService.isDefaultClient(true, nsIShellService.MAIL))
      window.openDialog("chrome://messenger/content/defaultClientDialog.xul", "DefaultClient",
                        "modal,centerscreen,chrome,resizable=no");

    var addrbookSession = Components.classes["@mozilla.org/addressbook/services/session;1"].
                            getService(Components.interfaces.nsIAddrBookSession);
    addrbookSession.addAddressBookListener(
      pbAddressBookChangeObserver,
      Components.interfaces.nsIAddrBookSession.added |
      Components.interfaces.nsIAddrBookSession.changed |
      Components.interfaces.nsIAddrBookSession.directoryItemRemoved);
    pbUnsentMessageMonitor.load();
  }

  setTimeout(completeStartup, 0);

  setTimeout("loadStartFolder(gStartFolderUri);", 0);

  // FIX ME - later we will be able to use onload from the overlay
  OnLoadMsgHeaderPane();

  gHaveLoadedMessage = false;

  gNotifyDefaultInboxLoadedOnStartup = true;

  //Set focus to the Thread Pane the first time the window is opened.
  SetFocusThreadPane();

  // initialize the customizeDone method on the customizeable toolbar
  var toolbox = document.getElementById("mail-toolbox2");
  toolbox.customizeDone = function(aEvent) { MailToolboxCustomizeDone(aEvent, "CustomizeMailToolbar"); };

  var toolbarset = document.getElementById('customToolbars');
  toolbox.toolbarset = toolbarset;

#ifdef XP_MACOSX
  let val = pref.getCharPref('pb.toolbar.customization.mode');
  pbUpdateToolbarMode('mail-bar7', val, pref.getCharPref('pb.toolbar.customization.iconsize'));
  document.getElementById('messengerWindow').setAttribute("color", val == 'monochrome-full' || val == 'monochrome-icons' ? 'mono' : '');
  GetFolderTree().treeBoxObject.clearStyleAndImageCaches();
  GetFolderTree().treeBoxObject.invalidate();
  GetAccountTree().treeBoxObject.clearStyleAndImageCaches();
  GetAccountTree().treeBoxObject.invalidate();
#endif

  initializeMailServices();

  // Toolkit has no easy way to disable quick find (where '/' brings up the quick find bar). We don't want that functionality
  // in Postbox as '/' maps to full text search. So force the find bar to unregister itself as a key listener
  let findBar = document.getElementById("FindToolbar");
  if (findBar && findBar._browser)
    findBar._browser.removeEventListener("keypress", findBar, false);

  // for add-ons that may require a status bar, reveal it if necessary
  let statusbar = document.getElementById('status-bar');
  let statusTextBox = document.getElementById('statusTextBox');
  if (statusbar && statusTextBox)
  {
    if (statusbar.childNodes.length + statusTextBox.childNodes.length > statusbar.getElementsByAttribute("postboxstatusbar","original").length +
        statusTextBox.getElementsByAttribute("postboxstatusbar","original").length)
      statusbar.removeAttribute('collapsed');
  }
}

function OnUnloadMessenger()
{
  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.removeObserver(gPb3PaneWindowObserver, "pb-validation-change");
  observerService.removeObserver(gPb3PaneWindowObserver, "pb-languages-loaded");
  observerService.removeObserver(gPb3PaneWindowObserver, "pb-label-changed");
  observerService.removeObserver(gPb3PaneWindowObserver, "quit-application-requested");
  
  window.removeEventListener("MozEnterNativeFullScreen", onEnterFullScreenMode);
  window.removeEventListener("MozExitNativeFullScreen", onExitFullScreenMode, false);
        
  gFocusPane.uninitFor3Pane();
  gFavoritesBar.uninitFor3Pane();
  pbUnsentMessageMonitor.unload();
  PersistTabs();
  OnLeavingFolder(gMsgFolderSelected);  // mark all read in current folder
  accountManager.removeIncomingServerListener(gThreePaneIncomingServerListener);
  document.getElementById('tabmail').closeTabs();

  gPhishingDetector.shutdown();

  // FIX ME - later we will be able to use onload from the overlay
  OnUnloadMsgHeaderPane();

  OnUnloadFolderPane();

  OnMailWindowUnload();
}

/**
 * @param aStartFolder if session store is empty or aInitialUri is true, StartUpWithTabs will open
 *         a new tab for aStartFolder.
 * @param aInitialUri the uri passed in via the command line
 */
function StartUpWithTabs(startFolder, aInitialUri)
{
  //  try
  {
    var sessionStore = Components.classes["@mozilla.org/mailnews/mailsessionstore;1"].getService(Components.interfaces.nsIMailSessionStore);
    var s = new Components.utils.Sandbox("about:blank");
    s.importFunction(MsgOpenNewTabForFolder);
    s.importFunction(MsgOpenNewTabForMessage);
    // not sure how to make this work dynamically for lightning
    if ("pbOpenNewCalendarTab" in window)
      s.importFunction(pbOpenNewCalendarTab);
     if ("pbOpenNewTaskTab" in window)
      s.importFunction(pbOpenNewTaskTab);
    var sessionString = sessionStore.getSessionState();
    var startFolderUri = startFolder.URI;
    // make sure we aren't going to open a folder tab for startFolder since we're going to open it already.
    if (aInitialUri)
    {
      var startFolderPersistStr = "MsgOpenNewTabForFolder(\"" + startFolderUri + "\", -1, false);";
      sessionString = sessionString ? sessionString.replace(startFolderPersistStr, "", "g") : '';
    }
    // if the first tab we are attempting to restore is not a folder tab, open a folder tab first.
    // see bug #5360 for details
    if (!sessionString || (aInitialUri && !startFolder.isServer) || sessionString.indexOf("MsgOpenNewTabForFolder"))
      MsgOpenNewTabForFolder(startFolderUri, -1, false);
    if (sessionString.length)
    {
      Components.utils.evalInSandbox(sessionString, s);
      document.getElementById('tabmail').mTabContainer.selectedIndex = 0;
    }
    // ensure there's always at least one tab! This could happen if there were errors restoring
    // the session state because the message or folder being restored in a tab may no longer exist.
    if (!document.getElementById('tabmail').tabOwners.length)
      MsgOpenNewTabForFolder(startFolderUri, -1, false);
  }
  //  catch (ex) {dump(ex); }
}

function PersistTabs()
{
  try
  {
    var sessionStore = Components.classes["@mozilla.org/mailnews/mailsessionstore;1"].getService(Components.interfaces.nsIMailSessionStore);
    var persistString = document.getElementById('tabmail').persistTabString();
    sessionStore.writeState(persistString);
  }
  catch (ex) {dump(ex); }
}

function NotifyObservers(aSubject, aTopic, aData)
{
  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.notifyObservers(aSubject, aTopic, aData);
}

function loadStartFolder(initialUri)
{
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage("loadStartFolder() with [" + initialUri + "]");

    var folderTree = GetFolderTree();
    var defaultServer = null;
    var startFolderResource = null;
    var isLoginAtStartUpEnabled = false;

    // Look to see if a master password is set, if so prompt for it to try
    // and avoid the multiple master password prompts on startup scenario.
    let token =
      Components.classes["@mozilla.org/security/pk11tokendb;1"]
                .getService(Components.interfaces.nsIPK11TokenDB)
                .getInternalKeyToken();

    // If an empty string is valid for the internal token, then we don't
    // have a master password, else, if it does, then try to login.
    try {
      // Only log in to the internal token if it is already initialized,
      if (!token.needsUserInit)
        token.login(false);
    }
    catch (ex) {
    // If user cancels an exception is expected. checkPassword also
    // seems to fail w/ mozmill.
    }

    //First get default account
    try
    {   
        if (initialUri)
            startFolderResource = RDF.GetResource(initialUri);
        else
        {
          try
          {
            var defaultAccount = accountManager.defaultAccount;

            defaultServer = defaultAccount.incomingServer;
            var rootMsgFolder = defaultServer.rootMsgFolder;

            startFolderResource = rootMsgFolder.QueryInterface(Components.interfaces.nsIRDFResource);
            
            // Enable check new mail once by turning checkmail pref 'on' to bring
            // all users to one plane. This allows all users to go to Inbox. User can
            // always go to server settings panel and turn off "Check for new mail at startup"
            if (!pref.getBoolPref(kMailCheckOncePrefName))
            {
                pref.setBoolPref(kMailCheckOncePrefName, true);
                defaultServer.loginAtStartUp = true;
            }

            // Get the user pref to see if the login at startup is enabled for default account
            isLoginAtStartUpEnabled = defaultServer.loginAtStartUp;

            // Get Inbox only if when we have to login
            if (isLoginAtStartUpEnabled)
            {
                //now find Inbox
                var outNumFolders = new Object();
                var inboxFolder = rootMsgFolder.getFoldersWithFlag(0x1000, 1, outNumFolders);
                if (!inboxFolder)
                {
                  consoleService.logStringMessage("loadStartFolder(), no inbox, bailing out early");
                  throw "no inbox";
                }
                startFolderResource = inboxFolder.QueryInterface(Components.interfaces.nsIRDFResource);
            }
          }
          catch (ex)
          {
            consoleService.logStringMessage("loadStartFolder(), failed to get startFolderResource, ex: " + ex);
          }
        }

        var startFolder = startFolderResource.QueryInterface(Components.interfaces.nsIMsgFolder);

        // it is possible we were given an initial uri and we need to subscribe or try to add
        // the folder. i.e. the user just clicked on a news folder they aren't subscribed to from a browser
        // the news url comes in here.

        try
        {
          // Perform biff on the server to check for new mail, except for imap
          // or a pop3 account that is deferred or deferred to,
          // or the case where initialUri is non-null (non-startup)
          if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder
              && !defaultServer.isDeferredTo &&
              defaultServer.rootFolder == defaultServer.rootMsgFolder)
            defaultServer.performBiff(msgWindow);
        }
        catch (ex)
        {
          consoleService.logStringMessage("Exception in loadStartFolder() when calling performBiff(), ex = " + ex);
        }
        
        consoleService.logStringMessage("calling StartUpWithTabs() with [" + startFolder.URI + "," + initialUri + "]");
        StartUpWithTabs(startFolder, initialUri);
    }
    catch(ex)
    {
      consoleService.logStringMessage("Exception in loadStartFolder(), ex = " + ex);
      // this is the case where we're trying to auto-subscribe to a folder.
      if (initialUri && !startFolder.parent)
      {
        consoleService.logStringMessage("loadStartFolder(), hack to force display of thread pane");
        // hack to force display of thread pane.
        ShowingThreadPane();
        // mscott: this might be broken after landing flying pigs bug #21 where I emoved loadURL
        messenger.openURL(initialUri);
        return;
      }
      consoleService.logStringMessage("If caused by no default account, we know about this.");
    }

    // open the tab for trial users in the background if we have a wizard (a first run case)
    // or pb.start_tab.last_day doesn't have  a value yet (a first run migration case )
    let openTrialTabInBackground = !gNoWizard || !gPrefBranch.prefHasUserValue('pb.start_tab.last_day');
    pbOpenTabForTrialUsers(openTrialTabInBackground);
  
    // if gLoadStartFolder is true, then we must have just created a POP3 account
    // and we aren't supposed to initially download mail. (Bug #270743)
    if (gLoadStartFolder)
      MsgGetMessagesForAllServers(defaultServer);

    if (MailOfflineMgr.isOnline()) {
      // Check if we shut down offline, and restarted online, in which case
      // we may have offline events to playback. Since this is not a pref
      // the user should set, it's not in mailnews.js, so we need a try catch.
      let playbackOfflineEvents = false;
      try {
        playbackOfflineEvents = gPrefBranch.getBoolPref("mailnews.playback_offline");
      }
      catch(ex) {}
      if (playbackOfflineEvents)
      {
        gPrefBranch.setBoolPref("mailnews.playback_offline", false);
        MailOfflineMgr.offlineManager.goOnline(false, true, msgWindow);
      }
    }
    // If appropriate, send unsent messages. This may end up prompting the user,
    // so we need to get it out of the flow of the normal load sequence.
    function checkUnsent() {
      if (MailOfflineMgr.isOnline() && MailOfflineMgr.shouldSendUnsentMessages())
        SendUnsentMessages();

      // All core modal dialogs are done, the user can now interact with the 3-pane window
      var obs = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
      obs.notifyObservers(window, "mail-startup-done", null);
    }
    setTimeout(checkUnsent, 0);

    InitWebServices();

    RecordUIAnalytics();
}

function InitWebServices()
{
    // initialize the web services.
    // Facebook, Twitter and LinkedIn download contacts
    // But we aren't in a hurry to do this so give it a delay.
    var delay  = gPrefBranch.getIntPref("pb.webservices.signon_delay");
    // XXX TODO
    // these all slam at the same time.  space them out?
    setTimeout("Components.classes['@facebook.com/facebook-service;1'].getService(Components.interfaces.fbIFacebookService).savedSessionStart();", delay);
    setTimeout("Components.classes['@postbox-inc.com/webservices/twitter;1'].getService(Components.interfaces.pbITwitterService).initTwitter(true, window, null);", delay);
    setTimeout("Components.classes['@postbox-inc.com/webservices/linkedin;1'].getService(Components.interfaces.pbILinkedInService).initLinkedIn(true, window, null);", delay);
    setTimeout("Components.classes['@postbox-inc.com/webservices/dropbox;1'].getService(Components.interfaces.pbIDropboxService).initDropbox(true, window, null, null);", delay);

#ifdef ENABLE_LANGUAGE_PACKS
    // initialize the languages
    setTimeout("Components.classes['@postbox-inc.com/languages;1'].getService(Components.interfaces.pbILanguageService);", 0);
#else
    // force general.useragent.locale to match the OS locale
    var currentLocale = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry).getSelectedLocale('global');
    if (pref.getBoolPref("intl.locale.matchOS") && pref.getCharPref("general.useragent.locale") != currentLocale)
      pref.setCharPref("general.useragent.locale", currentLocale);
#endif
}

function AddToSession()
{
  try {
    var mailSession = Components.classes[mailSessionContractID]
                                .getService(Components.interfaces.nsIMsgMailSession);
    var nsIFolderListener = Components.interfaces.nsIFolderListener;
    var notifyFlags = nsIFolderListener.added | nsIFolderListener.intPropertyChanged | nsIFolderListener.event | nsIFolderListener.propertyFlagChanged;
    mailSession.AddFolderListener(folderListener, notifyFlags);
  } catch (ex) {
    dump("Error adding to session\n");
  }
}

function InitPanes()
{
  OnLoadFolderPane();
  OnLoadThreadPane();
  SetFocusPaneVisibility(false);
  SetFolderPaneVisibility(false);
  SetInspectorPaneVisibility(false);
  SetupCommandUpdateHandlers();
}

function OnLoadFolderPane()
{
  // want to start up showing the folder pane, in case we shut down
  // with the 3-pane showing a single message.
  gAllMailRootFolder = new folderContainer(kAllAccountsURI, gMessengerBundle.getString('allAccountsFolderName'), 0, null);
  gAllMailFolders = new folderContainer(kAllMailFoldersURI, gMessengerBundle.getString('allMailFoldersFolderName'), 0, gAllMailRootFolder);

  var accountTree = document.getElementById('accountTree');
  gAccountView = new FolderTreeView();
  gAccountView.load(null, accountTree);
  gAccountView.addViewObserver(gAccountTreeViewObserver);
  accountTree.treeBoxObject.view = gAccountView.QueryInterface(Components.interfaces.nsITreeView);

  var folderTree = GetFolderTree();
  gFolderView = new FolderTreeView();
  // add our folder view observer
  gFolderView.addViewObserver(folderViewObserver);
  folderTree.treeBoxObject.view = gFolderView.QueryInterface(Components.interfaces.nsITreeView);

  folderTree.addEventListener("click",FolderPaneOnClick,true);
  folderTree.addEventListener("mousedown",TreeOnMouseDown,true);
  // register as a mousedown observer on the account tree to make dragging into groups work without
  // selecting the account.
  accountTree.addEventListener('mousedown', pbAccountTreeMouseDown, true);
  accountTree.addEventListener('mouseup',   pbDelayedAccountTreeSelect, true);

  // now that the trees have been initialized, update the focus pane headers
  // so any colapsed folder trees get their titles set correctly
  updateFocusPaneHeaders();

  // delay initializing the favorites bar for proper first run experience
  setTimeout(function() {gFavoritesBar.initFor3Pane();}, 0);
}

function OnUnloadFolderPane()
{
  gAccountView.close();
  gAccountView.removeViewObserver(gAccountTreeViewObserver);
  var folderTree = GetFolderTree();
  gFolderView.close();
  gFolderView.removeViewObserver(folderViewObserver);
  folderTree.removeEventListener("click",FolderPaneOnClick,true);
  folderTree.removeEventListener("mousedown",TreeOnMouseDown,true);
  var accountTree = document.getElementById('accountTree');
  accountTree.removeEventListener('mousedown', pbAccountTreeMouseDown, true);
  accountTree.removeEventListener('mouseup',   pbDelayedAccountTreeSelect, true);
  UnloadCommandUpdateHandlers();
}

// builds prior to 12-08-2001 did not have the labels column
// in the thread pane.  so if a user ran an old build, and then
// upgraded, they get the new column, and this causes problems.
// We're trying to avoid a similar problem to bug #96979.
// to work around this, we hide the column once, using the
// "mailnews.ui.threadpane.version" pref.
function UpgradeThreadPaneUI()
{
  var threadPaneUIVersion;

  try {

    threadPaneUIVersion = pref.getIntPref("mailnews.ui.threadpane.version");

    // Note: threadTree._reorderColumn will throw an ERROR if the columns specified are already in the same order!

    if (threadPaneUIVersion < 6)
    {
      var threadTree = document.getElementById("threadTree");
      var dateCol = document.getElementById("dateCol");
      var receivedCol = document.getElementById("receivedCol");
      var junkCol = document.getElementById("junkStatusCol");

      if (threadPaneUIVersion < 5)
      {
        if (threadPaneUIVersion < 4)
        {
          if (threadPaneUIVersion < 3)
          {

            // in thunderbird, we are inserting the junk column just before the
            // date column.
            threadTree._reorderColumn(junkCol, dateCol, true);
          }

          var senderCol = document.getElementById("senderCol");
          var recipientCol = document.getElementById("recipientCol");
          threadTree._reorderColumn(recipientCol, junkCol, true);
          threadTree._reorderColumn(senderCol, recipientCol, true);

        } // version 4 upgrades

        // version 5 adds a new column called attachments
        var attachmentCol = document.getElementById("attachmentCol");
        var subjectCol = document.getElementById("subjectCol");

        threadTree._reorderColumn(attachmentCol, subjectCol, true);

      } // version 5 upgrades

      if (dateCol)
        threadTree._reorderColumn(receivedCol, dateCol, true);
      else
        threadTree._reorderColumn(receivedCol, junkCol, false);

      pref.setIntPref("mailnews.ui.threadpane.version", 6);

    } // version 6 upgrades
  }
  catch (ex) {
    dump("UpgradeThreadPane: ex = " + ex + "\n");
  }
}

function OnLoadThreadPane()
{
  UpgradeThreadPaneUI();
}

/* Functions for accessing particular parts of the window*/
function GetAccountTree()
{
  if (!gAccountTree)
    gAccountTree = document.getElementById("accountTree");
  return gAccountTree;
}

function GetFolderTree()
{
  if (! gFolderTree)
    gFolderTree = document.getElementById("folderTree");
  return gFolderTree;
}

function GetMessagePane()
{
  if (!gMessagePane)
    gMessagePane = document.getElementById("messagepanebox");
  return gMessagePane;
}

function getMailToolbox()
{
  return document.getElementById("mail-toolbox2");
}

function FindInSidebar(currentWindow, id)
{
  var item = currentWindow.document.getElementById(id);
  if(item)
    return item;

  for(var i = 0; i < currentWindow.frames.length; i++)
  {
    var frameItem = FindInSidebar(currentWindow.frames[i], id);
    if(frameItem)
      return frameItem;
  }

  return null;
}

function GetThreadAndMessagePaneSplitter()
{
  if(gThreadAndMessagePaneSplitter) return gThreadAndMessagePaneSplitter;
  var splitter = document.getElementById('threadpane-splitter');
  gThreadAndMessagePaneSplitter = splitter;
  return splitter;
}

function GetUnreadCountElement()
{
  if(gUnreadCount) return gUnreadCount;
  var unreadCount = document.getElementById('unreadMessageCount');
  gUnreadCount = unreadCount;
  return unreadCount;
}
function GetTotalCountElement()
{
  if(gTotalCount) return gTotalCount;
  var totalCount = document.getElementById('totalMessageCount');
  gTotalCount = totalCount;
  return totalCount;
}

function IsMessagePaneCollapsed()
{
  return GetMessagePane().collapsed;
}

function IsFolderPaneCollapsed()
{
  return GetFolderTree().parentNode.collapsed;
}

function FindMessenger()
{
  return messenger;
}

function ClearThreadPaneSelection()
{
  try {
    if (gDBView) {
      var treeView = gDBView.QueryInterface(Components.interfaces.nsITreeView);
      var treeSelection = treeView.selection;
      if (treeSelection)
        treeSelection.clearSelection();
      gDBView.clearDisplayedMessage();
    }
  }
  catch (ex) {
    dump("ClearThreadPaneSelection: ex = " + ex + "\n");
  }
}

function GetSelectedAccountIndex()
{
  return _getSelectedTreeIndex(GetAccountTree());
}

function GetSelectedFolderIndex()
{
  return _getSelectedTreeIndex(GetFolderTree());
}

function _getSelectedTreeIndex(aTree)
{
  var startIndex = {};
  var endIndex = {};
  aTree.view.selection.getRangeAt(0, startIndex, endIndex);
  return startIndex.value;
}

// Function to change the highlighted row to where the mouse was clicked
// without loading the contents of the selected row.
// It will also keep the outline/dotted line in the original row.
function ChangeSelectionWithoutContentLoad(event, tree)
{
    var treeBoxObj = tree.treeBoxObject;
    var treeSelection = treeBoxObj.view.selection;

    var row = treeBoxObj.getRowAt(event.clientX, event.clientY);
    // make sure that row.value is valid so that it doesn't mess up
    // the call to ensureRowIsVisible().
    if((row >= 0) && !treeSelection.isSelected(row))
    {
      var saveCurrentIndex = treeSelection.currentIndex;
      treeSelection.selectEventsSuppressed = true;
      // include children in the thread pane when the selected row is
      // a container row with children
      if (tree.id == "threadTree")
      {
        var dbview = GetDBView();
        var adjustedRow = row;
        // advance row past any rich rows if a rich tree
        while (tree.pbRichTree && !dbview.isSelectable(adjustedRow, null) && adjustedRow < dbview.rowCount)
          adjustedRow++;
        if (dbview.isContainer(adjustedRow) && dbview.isContainerOpen(adjustedRow) && !dbview.isContainerEmpty(adjustedRow))
        {
          // extend selection to include the children
          dbview.ExpandAndSelectThreadByIndex(adjustedRow, false); // this will perform the ranged select for us
        }
        else
          treeSelection.select(row);
      }
      else
        treeSelection.select(row);
      treeSelection.currentIndex = saveCurrentIndex;
      treeBoxObj.ensureRowIsVisible(row);
      treeSelection.selectEventsSuppressed = false;

      // Keep track of which row in the thread pane is currently selected.
      if(tree.id == "threadTree")
        gThreadPaneCurrentSelectedIndex = row;
    }
    event.stopPropagation();
}

function TreeOnMouseDown(event)
{
    // Detect right mouse click and change the highlight to the row
    // where the click happened without loading the message headers in
    // the Folder or Thread Pane.
    if (event.button == 2)
    {
      gRightMouseButtonDown = true;
      // ignore tree clicks for things like treecols
      if (event.target.localName == 'treechildren')
        ChangeSelectionWithoutContentLoad(event, event.target.parentNode);
    }
    else
      gRightMouseButtonDown = false;
}

function FolderPaneOnClick(event)
{
    // we only care about button 0 (left click) events
    if (event.button != 0)
        return;

    var folderTree = GetFolderTree();
    var row = {};
    var col = {};
    var elt = {};
    folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);
    if (row.value == -1) {
      if (event.originalTarget.localName == "treecol") {
        // clicking on the name column in the folder pane should not sort
        event.stopPropagation();
      }
    }
    else if ((event.originalTarget.localName == "slider") ||
             (event.originalTarget.localName == "scrollbarbutton")) {
      event.stopPropagation();
    }
    else if ((event.detail == 2) && (elt.value != "twisty") &&
             (folderTree.view.getLevel(row.value) != 0)) {
      FolderPaneDoubleClick(row.value, event);
    }
}

function FolderPaneDoubleClick(folderIndex, event)
{
    if (!pref.getBoolPref("mailnews.reuse_thread_window2"))
    {
      var folderResource = GetFolderResource(GetFolderTree(), folderIndex);
      // Open a new folder tab only if we are double clicking on
      // folders or newsgroups.
      MsgOpenNewTabForFolderWithAnimation(folderResource.Value, -1 /* key */, true /* background */, false, GetSelectedAccountURI());

      // double clicking should not toggle the open / close state of the
      // folder.  this will happen if we don't prevent the event from
      // bubbling to the default handler in tree.xml
      event.stopPropagation();
    }
}

function ChangeSelection(tree, newIndex)
{
    if(newIndex >= 0)
    {
      tree.view.selection.select(newIndex);
      tree.treeBoxObject.ensureRowIsVisible(newIndex);
    }
}

function GetSelectedFolders()
{
    var folderArray = [];
    var k = 0;
    var folderTree = GetFolderTree();
    if (!folderTree)
      return folderArray;
    var rangeCount = folderTree.view.selection.getRangeCount();

    for(var i = 0; i < rangeCount; i++)
    {
        var startIndex = {};
        var endIndex = {};
        folderTree.view.selection.getRangeAt(i, startIndex, endIndex);
        for (var j = startIndex.value; j <= endIndex.value; j++)
        {
            var folderResource = GetFolderResource(folderTree, j);
            folderArray[k++] = folderResource.Value;
        }
    }

    return folderArray;
}

function GetSelectedMsgFolders()
{
    var folderArray = [];
    var k = 0;
    var folderTree = GetFolderTree();
    if (!folderTree)
      return folderArray;
    var rangeCount = folderTree.view.selection.getRangeCount();

    for(var i = 0; i < rangeCount; i++)
    {
        var startIndex = {};
        var endIndex = {};
        folderTree.view.selection.getRangeAt(i, startIndex, endIndex);
        for (var j = startIndex.value; j <= endIndex.value; j++)
        {
          var folderResource = GetFolderResource(folderTree, j);
          if (folderResource instanceof Components.interfaces.nsIMsgFolder)
            folderArray[k++] = folderResource;
        }
    }
    return folderArray;
}

function GetFirstSelectedMessage()
{
    try {
        return gDBView.URIForFirstSelectedMessage;
    }
    catch (ex) {
        return null;
    }
}

function GetSelectedIndices(dbView)
{
  if (!dbView)
    return null;
  try {
    var indicesArray = {};
    var length = {};
    dbView.getIndicesForSelection(indicesArray,length);
    return indicesArray.value;
  }
  catch (ex) {
    dump("GetSelectedIndices ex = " + ex + "\n");
    return null;
  }
}

function GetSelectedMessages()
{
  try {
    var messageArray = {};
    var length = {};
    var view = GetDBView();
    view.getURIsForSelection(messageArray,length);
    if (length.value)
      return messageArray.value;
    else
      return null;
  }
  catch (ex) {
    dump("GetSelectedMessages ex = " + ex + "\n");
    return null;
  }
}

function GetLoadedMsgFolder()
{
    if (!gDBView) return null;
    return gDBView.msgFolder;
}

function GetLoadedMessage()
{
    try {
        return gDBView.URIForFirstSelectedMessage;
    }
    catch (ex) {
        return null;
    }
}

//Clear everything related to the current message. called after load start page.
function ClearMessageSelection()
{
  ClearThreadPaneSelection();
}

// Figures out how many messages are selected (hilighted - does not necessarily
// have the dotted outline) above a given index row value in the thread pane.
function NumberOfSelectedMessagesAboveCurrentIndex(index)
{
  var numberOfMessages = 0;
  var indicies = GetSelectedIndices(gDBView);

  if (indicies && indicies.length)
  {
    for (var i = 0; i < indicies.length; i++)
    {
      if (indicies[i] < index)
        ++numberOfMessages;
      else
        break;
    }
  }
  return numberOfMessages;
}

function SetNextMessageAfterDelete()
{
  var treeSelection = GetThreadTree().view.selection;

  if (treeSelection.isSelected(treeSelection.currentIndex))
  {
    gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete;
    gSelectedIndexWhenDeleting = treeSelection.currentIndex;
  }
  else if(gDBView.removeRowOnMoveOrDelete)
  {
    // Only set gThreadPaneDeleteOrMoveOccurred to true if the message was
    // truly moved to the trash or deleted, as opposed to an IMAP delete
    // (where it is only "marked as deleted".  This will prevent bug 142065.
    //
    // If it's an IMAP delete, then just set gNextMessageViewIndexAfterDelete
    // to treeSelection.currentIndex (where the outline is at) because nothing
    // was moved or deleted from the folder.
    gThreadPaneDeleteOrMoveOccurred = true;
    gNextMessageViewIndexAfterDelete = treeSelection.currentIndex - NumberOfSelectedMessagesAboveCurrentIndex(treeSelection.currentIndex);
  }
  else
    gNextMessageViewIndexAfterDelete = treeSelection.currentIndex;
}

/*
 * Ensure aItem is in the view.
 * if aItem is a nsIMsgFolder, we'll recursively ensure the parent of aItem is open and visible.
 * @return the corresponding view index for aItem. -1 if we were unable to ensure the item is visible in the view
 */
var _skipDefaultFolderLoad = false; // hacky! private variable used between EnsureFolderIndex and AccountSelectionChanged
function EnsureFolderIndex(aItem)
{
  // first, ensure the correct account is selected
  var selectedAccountIndex = GetSelectedAccountIndex();
  let selectedRootFolder = selectedAccountIndex != -1 ? gAccountView.getItemForViewIndex(selectedAccountIndex) : '';
  let newRootFolderToSelect = aItem.rootFolder;
  // We'll need to adjust the account tree if any of the following are true:
  // (1) if nothing is selected
  // (2) if the selected server is a group and aItem is not visible in that group
  // (3) if the selected root is not equal to aItem.rootFolder and a smart folder is not selected
  // (4) if the selected root is kAllAccountsURI and the selected root is not equal to aItem.rootFolder 
  if (selectedAccountIndex == -1 ||
      (selectedRootFolder.URI != kAllAccountsURI && SmartServerUtils.isSmartServer(selectedRootFolder.server) && gFolderView.ensureFolderIndex(aItem) == -1) ||
      (selectedRootFolder != newRootFolderToSelect && selectedRootFolder.URI != kAllAccountsURI && !SmartServerUtils.isSmartServer(selectedRootFolder.server)) ||
      (selectedRootFolder.URI == kAllAccountsURI && selectedRootFolder != newRootFolderToSelect))
  {
    _skipDefaultFolderLoad = true;
    // suppressing the selection event on the account tree isn't stopping AccountSelectionChange from being called, but it should!
    // if we can't find aItem.rootFolder then it must be in a collapsed group, so expand the appropriate group
    if (gAccountView.getViewIndexForItem(newRootFolderToSelect) == -1)
    {
      // find the smart server the rootFolder is a part of and expand it
      let server = SmartServerUtils.findSmartServerForServer(aItem.server);
      if (server)
      {
        // If the group only contains one server (aItem.server) - prefer selection of the group over aItem.server so we can avoid
        // toggling open a group with only one server.
        if (SmartServerUtils.getSortedAccountsForSmartServer(server).length > 1)
          gAccountView.toggleOpenState(gAccountView.getViewIndexForItem(server.rootFolder));
        else
          newRootFolderToSelect = server.rootFolder; // override the attempt to select aItem.rootFolder by selecting the group
      }
    }
    ChangeSelection(GetAccountTree(), gAccountView.getViewIndexForItem(newRootFolderToSelect));
    _skipDefaultFolderLoad = false;
  }

  // now that we know the right account is in the folder pane...
  return gFolderView.ensureFolderIndex(aItem);
}

function SelectFolder(aFolderUri)
{
  var folderTree = GetFolderTree();
  let folderItem;
  if (aFolderUri == kAllMailFoldersURI)
    folderItem = gAllMailFolders;
  else
    folderItem = GetMsgFolderFromUri(aFolderUri);

  // before we can select a folder, we need to make sure it is "visible"
  // in the tree.  to do that, we need to ensure that all its
  // ancestors are expanded
  ChangeSelection(folderTree, EnsureFolderIndex(folderItem));
}

function SelectMessage(messageUri)
{
  var msgHdr = messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri);
  if (msgHdr)
    gDBView.selectMsgByKey(msgHdr.messageKey);
}

function ReloadMessage()
{
  gDBView.reloadMessage();
}

function GetDBView()
{
  return gDBView;
}

/**
 * FIX ME - this should be GetSelectedFolder or something like that since we
 * are returning the selected folder.
 */
function GetFolderResource(tree, index)
{
  return (index >= 0) ? gFolderView.getItemForViewIndex(index) : null;
}

function LoadNavigatedToMessage(msgHdr, folder, folderUri)
{
  if (IsCurrentLoadedFolder(folder))
  {
    gDBView.selectMsgByKey(msgHdr.messageKey);
  }
  else
  {
    gStartMsgKey = msgHdr.messageKey;
    SelectFolder(folderUri);
  }
}

// Some of the per account junk mail settings have been
// converted to global prefs. Let's try to migrate some
// of those settings from the default account.
function MigrateJunkMailSettings()
{
  var junkMailSettingsVersion = pref.getIntPref("mail.spam.version");
  if (!junkMailSettingsVersion)
  {
    // get the default account, check to see if we have values for our
    // globally migrated prefs.
    var defaultAccount;
    try {
      defaultAccount = accountManager.defaultAccount;
    } catch (ex) {}
    if (defaultAccount && defaultAccount.incomingServer)
    {
      // we only care about
      var prefix = "mail.server." + defaultAccount.incomingServer.key + ".";
      if (pref.prefHasUserValue(prefix + "manualMark"))
        pref.setBoolPref("mail.spam.manualMark", pref.getBoolPref(prefix + "manualMark"));
      if (pref.prefHasUserValue(prefix + "manualMarkMode"))
        pref.setIntPref("mail.spam.manualMarkMode", pref.getIntPref(prefix + "manualMarkMode"));
      if (pref.prefHasUserValue(prefix + "spamLoggingEnabled"))
        pref.setBoolPref("mail.spam.logging.enabled", pref.getBoolPref(prefix + "spamLoggingEnabled"));
      if (pref.prefHasUserValue(prefix + "markAsReadOnSpam"))
        pref.setBoolPref("mail.spam.markAsReadOnSpam", pref.getBoolPref(prefix + "markAsReadOnSpam"));
    }
    // bump the version so we don't bother doing this again.
    pref.setIntPref("mail.spam.version", 1);
  }
}

// Thunderbird has been storing old attachment download meta data in downloads.rdf
// even though there was no way to show or clean up this data. Now that we are using
// the new download manager in toolkit, we don't want to present this old data.
// To migrate to the new download manager, remove downloads.rdf.
function MigrateAttachmentDownloadStore()
{
  var attachmentStoreVersion = pref.getIntPref("mail.attachment.store.version");
  if (!attachmentStoreVersion)
  {
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties);
    var downloadsFile = dirService.get("DLoads", Components.interfaces.nsIFile);
    if (downloadsFile && downloadsFile.exists())
      downloadsFile.remove(false);

    // bump the version so we don't bother doing this again.
    pref.setIntPref("mail.attachment.store.version", 1);
  }
}

/**
 * example use:
 *   suggestUniqueFileName("testname",".txt",["testname", "testname1"])
 *   returns "testname2"
 * does not check file system for existing files
 * @param existingNames array of names in use
 */
function suggestUniqueFileName(identifier, type, existingNames) {
  let suffix = 1;
  let suggestion;
  let base = identifier;
  let exists;
  do {
    exists = false;
    suggestion = GenerateValidFilename(base, type);
    for (let i = 0; i < existingNames.length; i++) {
      if (existingNames[i] == suggestion) {
        base = identifier + suffix;
        suffix++;
        exists = true;
        break;
      }
    }
  } while (exists);
  return suggestion;
}

#include pbMsgMail3PaneWindow.js
